//LabPlot : SmoothListDialog.cc

#include "SmoothListDialog.h"
#include "defs.h"

SmoothListDialog::SmoothListDialog(MainWin *m, const char *name)
	: ListDialog(m, name)
{
	setCaption(i18n("Smooth Dialog"));
	KConfig *config = mw->Config();
	config->setGroup( "Smooth" );

	QTabWidget *tw = new QTabWidget(vbox);
	QVBox *tab1 = new QVBox(tw);

	new QLabel(i18n(" Smooth over n points :"),tab1);
	QHBox *hb = new QHBox(tab1);
	new QLabel(QString(" n = "),hb);
	nni = new KIntNumInput(config->readNumEntry("N",3),hb);
	nni->setRange(1,INF,1,false);

	Style *style=0;
	Symbol *symbol=0;
	QVBox *styletab;
	if(p && p->getPlot(p->API())->Type() == PSURFACE)
		styletab = surfaceStyle(tw,true);
	else
		styletab = simpleStyle(tw, style, symbol);

	tw->addTab(tab1,i18n("Parameter"));
	tw->addTab(styletab,i18n("Style"));

	QObject::connect(ok,SIGNAL(clicked()),SLOT(ok_clicked()));
	QObject::connect(apply,SIGNAL(clicked()),SLOT(apply_clicked()));
	QObject::connect(save,SIGNAL(clicked()),SLOT(saveSettings()));

	setMinimumWidth(vbox->minimumSizeHint().width());
	setMinimumHeight(gbox->minimumSizeHint().height()+vbox->minimumSizeHint().height());
	resize(minimumSize());
}

void SmoothListDialog::saveSettings() {
	KConfig *config = mw->Config();
	config->setGroup( "Smooth" );

	config->writeEntry("N", nni->value());
}

int SmoothListDialog::apply_clicked() {
	int n = nni->value();

	if(!(n % 2)) {
		KMessageBox::error(this,i18n("n is even!\nPlease choose an uneven number."));
		return -1;
	}

	if(s) {
		QTable *table = s->Table();
		int col = table->currentColumn();

		if(n >= table->numRows()) {
			KMessageBox::error(this,i18n("n bigger than number of rows!\nPlease use a smaller value."));
			return -1;
		}

		// add column if needed
		bool empty=true;
		for(int i=0;i<table->numRows();i++) {
			if(!table->text(i,table->numCols()-1).isEmpty()) {
				empty =false;
			}
		}
		if (!empty)
			s->addColumn();

		int newcol = table->numCols()-1;
		for (int i = 0;i<table->numRows()-n+1;i++) {
			double sum = 0;
			for (int j=0;j<n;j++) {
				sum+=table->text(i+j,col).toDouble();
			}
			sum = sum/n;

			// insert into last column
			table->setText(i+n/2,newcol,QString::number(sum));
		}
		return 0;
	}

	GraphList *gl = p->getPlot(p->API())->getGraphList();
	if(gl->Number()==0) {
		KMessageBox::error(this,i18n("No graph found!"));
		return -2;
	}
	int item = (int) (lv->itemPos(lv->currentItem())/lv->currentItem()->height());

	GRAPHType st = gl->getType(item);

	Style *style=0;
	Symbol *symbol=0;

	if(st != GRAPHM) {
		style = new Style((StylesType)cb2->currentItem(),color->color(),filled->isChecked(),fcolor->color(),
			width->value(),pencb->currentItem(),brushcb->currentItem());
		style->setBoxWidth(boxwidth->value());
		style->setAutoBoxWidth(autobox->isChecked());
		style->setPointsSorting(sortpointscb->isChecked());
		symbol = new Symbol((SType)symbolcb->currentItem(),scolor->color(),ssize->value(),
			(FType)symbolfillcb->currentItem(),sfcolor->color(),sbrushcb->currentItem());
	}

	if (st == GRAPH2D) {
		Graph2D *g = gl->getGraph2D(item);
		int nx = g->Number();
		Point *ptr = new Point[nx];
		Point *a = g->Data();
		QString label = g->getLabel()->simpleTitle();

		if(n >= nx) {
			KMessageBox::error(this,i18n("n bigger than number of points!\nPlease use a smaller value."));
			return -1;
		}

		// do the smooth
		double xmin=0, xmax=1, ymin=0, ymax=1;
		for (int i = 0;i<nx-n+1;i++) {
			double x = a[(int) (i+n/2)].X();
			double y = 0;
			for (int j=0;j<n;j++) {
				y+=a[i+j].Y();
			}
			y = y/n;

			ptr[i].setPoint(x,y);
		}

		mw->calculateRanges2D(ptr,nx-n+1,&xmin,&xmax,&ymin,&ymax);

		// create the new Graph
		LRange range[2];
		range[0] = LRange(xmin,xmax);
		range[1] = LRange(ymin,ymax);

		QString fun = QString(i18n("smooth (")+QString::number(n)+i18n(") of ")+label);

		Graph2D *ng = new Graph2D(fun,fun,range,SSPREADSHEET,P2D,style,symbol,ptr,nx-n+1);
		mw->addGraph2D(ng,sheetcb->currentItem());
	}
	else if (st == GRAPH3D) {
		Graph3D *g = gl->getGraph3D(item);
		int nx = g->Number();
		QString label = g->getLabel()->simpleTitle();
		Point3D *ptr = new Point3D[nx];
		Point3D *a = g->Data();

		if(n >= nx) {
			KMessageBox::error(this,i18n("n bigger than number of points!\nPlease use a smaller value."));
			return -1;
		}

		// do the average
		double xmin=0, xmax=1, ymin=0, ymax=1, zmin=0, zmax=1;
		for (int i = 0;i<nx-n+1;i++) {
			double x = a[(int) (i+n/2)].X();
			double y = 0, z = 0;
			if (type == P2D) {	// 2derr
				for (int j=0;j<n;j++) {
					y+=a[i+j].Y();
					z+=a[i+j].Z();
				}
				y = y/n;
				z = z/n;
			}
			else {		// 3d
				//TODO
				KMessageBox::error(this,i18n("Sorry. This function is not yet implemented!"));
				return -2;
			}

			ptr[i].setPoint(x,y,z);
		}

		mw->calculateRanges3D(ptr,nx-n+1,&xmin,&xmax,&ymin,&ymax,&zmin,&zmax);

		// create the new Graph
		LRange range[3];
		range[0] = LRange(xmin,xmax);
		range[1] = LRange(ymin,ymax);
		range[2] = LRange(zmin,zmax);

		QString fun = QString(i18n("smooth (")+QString::number(n)+i18n(") of ")+g->getLabel()->simpleTitle());

		Graph3D *ng = new Graph3D(fun,fun,range,SSPREADSHEET,type,style,symbol,ptr,nx-n+1,1);
		ng->setNumberX((int)(g->NX()-n));
		ng->setNumberY((int)(g->NY()));
		mw->addGraph3D(ng,sheetcb->currentItem());
	}
	else if (st == GRAPH4D) {
		Graph4D *g = gl->getGraph4D(item);
		bool gtype = g->Type();
		int nx = g->Number();
		Point4D *ptr = new Point4D[nx];
		Point4D *a = g->Data();

		if(n >= nx) {
			KMessageBox::error(this,i18n("n bigger than number of points!\nPlease use a smaller value."));
			return -1;
		}

		double xmin=0, xmax=1, ymin=0, ymax=1, zmin=0, zmax=1, tmin=0, tmax=1;
		for (int i = 0;i<nx-n+1;i++) {
			double x = a[(int) (i+n/2)].X();
			double y = 0, z = 0,t = 0;
			if (type == P2D) {	// 2derr
				for (int j=0;j<n;j++) {
					y+=a[i+j].Y();
					z+=a[i+j].Z();
					t+=a[i+j].T();
				}
				y = y/n;
				z = z/n;
				t = t/n;
			}

			ptr[i].setPoint(x,y,z,t);
		}

		mw->calculateRanges4D(ptr,nx-n+1,&xmin,&xmax,&ymin,&ymax,&zmin,&zmax,&tmin,&tmax);

		// create the new Graph
		LRange range[4];
		range[0] = LRange(xmin,xmax);
		range[1] = LRange(ymin,ymax);
		range[2] = LRange(zmin,zmax);
		range[3] = LRange(tmin,tmax);

		QString fun = QString(i18n("smooth (")+QString::number(n)+i18n(") of ")+g->getLabel()->simpleTitle());

		Graph4D *ng = new Graph4D(fun,fun,range,SSPREADSHEET,type,style,symbol,ptr,nx-n+1,gtype);
		mw->addGraph4D(ng,sheetcb->currentItem());
	}
	else if (st == GRAPHM) {		// PSURFACE and P3D
		//TODO
		KMessageBox::error(this,i18n("Sorry. This function is not yet implemented!"));
		return -3;
	}

	updateList();

	return 0;
}
