// LabPlot : Spreadsheet.cc

#include <stdlib.h>
#include <time.h>
#ifdef HAVE_SGI_STL
#include <vector>
#endif

#ifdef HAVE_SOLARIS
#include <ieeefp.h>
#endif

#include <qtable.h>
#include <qfile.h>
#include <qhbox.h>
#include <qcursor.h>
#include <qclipboard.h>
#include <qinputdialog.h>

#include <kmessagebox.h>
#include <kiconloader.h>
#include <kconfig.h>
#include <ktempfile.h>
#include <kdeversion.h>
#if KDE_VERSION >= 0x030200
#include <kinputdialog.h>
#endif

#include "Spreadsheet.h"
#include "SpreadsheetValuesDialog.h"
#include "SpreadsheetPropertiesDialog.h"
#include "TestsRListDialog.h"
#include "ListDialog.h"
#include "ExportDialog.h"
#include "DestinationDialog.h"
#include "LTableItem.h"
#include "Plot2DSurface.h"

#include "spreadsheetProperties.h"
#include "source.h"
#include "defs.h"
#include "parser_extern.h"
#include "pixmaps/pixmap.h"

#ifdef HAVE_GSL
#include <gsl/gsl_statistics.h>
#endif

#ifdef USE_NEW_TABLE
//#include <vector>
#endif

#include <exception>
using namespace std;

Spreadsheet::Spreadsheet(QWidget *p, MainWin *m, const char *name)
	:QWidget(p,name), mw(m)
{
	kdDebug()<<"Spreadsheet()"<<endl;

	ld=0;
	graph=0;
	gtype=GRAPH2D;

	widgettype = WSPREADSHEET;

	// find free worksheet number
	QWidgetList list = mw->getWorkspace()->windowList();
	int nr=0, sheets = mw->NrWorksheets()+mw->NrSpreadsheets();
	bool found_free=false;
	kdDebug()<<"	Number of sheets "<<sheets<<endl;
	while(nr <= sheets && !found_free ) {
		nr++;
		found_free=true;
		for(int i=0;i<sheets;i++) {
			if(list.at(i) && list.at(i)->caption() == QString(i18n("Spreadsheet")+QString(" ")+QString::number(nr)))
				found_free=false;
		}
	}

	kdDebug()<<"	Using number "<<nr<<endl;
	title = i18n("Spreadsheet")+QString(" ")+QString::number(nr);
	setCaption(title);

	int NR=100;
#ifdef USE_NEW_TABLE
	table = new LTable(NR, 2, this );
#else
	table = new QTable( NR, 2, this );
#endif
	table->setRowMovingEnabled (true);
	table->setColumnMovingEnabled (true);
	for(int i=0;i<NR;i++) {
		LTableItem *item1 = new LTableItem( table, QTableItem::OnTyping,"");
		LTableItem *item2 = new LTableItem( table, QTableItem::OnTyping,"");
		table->setItem(i, 0, item1);
		table->setItem(i, 1, item2);
	}

	table->horizontalHeader()->setLabel( 0, QString( "A ")+i18n("{double}")+" [X]");
	table->horizontalHeader()->setLabel( 1, QString( "B ")+i18n("{double}")+" [Y]");
	table->horizontalHeader()->installEventFilter(this );

	table->setFocusPolicy(QWidget::StrongFocus);
	table->setFocus();
	QObject::connect(table,SIGNAL(currentChanged(int,int)),this,SLOT(updateValuesDialog()));
	QObject::connect(table,SIGNAL(selectionChanged()),this,SLOT(selectionChanged()));

	resize(table->sizeHint());
	show();
	if(mw) mw->updateSheetList();

	destination=-1;
	values_dialog=0;
	datafile=QString("");
}

void Spreadsheet::updateValuesDialog() {
	if(values_dialog != 0)
		values_dialog->updateDialog();
}

void Spreadsheet::selectionChanged() {
	if(mw->test_dialog != 0)
		(mw->test_dialog)->updateDialog();
}

// used for properties dialog on header
bool Spreadsheet::eventFilter(QObject *object, QEvent *e) {
//	kdDebug()<<"Spreadsheet::eventFilter()"<<endl;

	switch(e->type()) {
	case QEvent::MouseButtonDblClick:
		if(table->currentColumn() == -1 || ((QMouseEvent *) e)->x() > table->horizontalHeader()->headerWidth())
			addColumn();
		else
			(new SpreadsheetPropertiesDialog(mw,table,caption()))->show(); break;
/*	case QEvent::MouseButtonPress:
		kdDebug()<<"mouse press"<<endl;
		//table->clearSelection();
		//table->selectColumn(-1);
		return true;
		break;
*/
	default: break;
	}

	return QObject::eventFilter(object,e);
}

//! used for left upper corner
void Spreadsheet::mousePressEvent ( QMouseEvent * e ) {
	kdDebug()<<"mousePressEvent() @ x/y "<<e->x()<<' '<<e->y()<<endl;
	if(e->x()<100 && e->y()<25)		// left upper corner
		for (int i=0;i<table->numCols();i++)
			table->selectColumn(i);
}

//! clear spreadsheet
void Spreadsheet::Clear() {
	kdDebug()<<"Spreadsheet::Clear()"<<endl;
	for (int i=0;i<table->numCols();i++) {
		for (int j=0;j<table->numRows();j++) {
			if(table->numSelections() == 0 || table->isSelected(j,i))
				setText(j,i,"");
		}
	}
	table->clearSelection();
}

QStringList Spreadsheet::Info() {
	kdDebug()<<"Spreadsheet::Info()"<<endl;
	QStringList s;
	s<<i18n("SIZE:")<<QString::number(table->numCols())<<i18n(" x ")<<QString::number(table->numRows());

	return s;
}

//! set text of table item (i,j) to text
void Spreadsheet::setText(int i, int j, QString text) {
	LTableItem *item = new LTableItem( table, QTableItem::OnTyping,text);
	table->setItem(i, j, item);
}

void Spreadsheet::setCurrentColumn(int i) {
	table->clearSelection();
	table->setCurrentCell(0,i);
}

//! calculate number of filled rows
int Spreadsheet::filledRows(int col) {
	if(col < 0 || col > table->numCols())
		return 0;

	int row=0;
	for(int i=0;i<table->numRows();i++)
		if (!table->text(i,col).isEmpty())
			row++;
	return row;
}

//! calculate number of selected cols
int Spreadsheet::selectedColumns() {
	int nr=0;

	for(int j=0;j<table->numCols();j++)
		if(table->isColumnSelected(j))
			nr++;

	return nr;
}

//! calculate number of selected rows
int Spreadsheet::selectedRows() {
	int nr=0;

	for(int j=0;j<table->numRows();j++)
		if(table->isRowSelected(j))
			nr++;

	return nr;
}
	
int Spreadsheet::getColumnIndex(QString title) {
	for(int i=0;i<table->numCols();i++) {
		if (columnTitle(i) == title)
			return i;
	}

	return -1;
}

void Spreadsheet::resizeEvent(QResizeEvent *e) {
	table->resize(e->size());
}

void Spreadsheet::closeNow() {
	if (ld) updateGraphList();

	this->deleteLater();
	mw->updateSheetList();	// but this spread still exists!
}

void Spreadsheet::closeEvent(QCloseEvent *e) {
	kdDebug()<<"Spreadsheet::closeEvent()"<<endl;

	closeNow();
	e->accept();
}

// used when editing data
void Spreadsheet::updateGraphList() {
	kdDebug()<<"Spreadsheet::updateGraphList()"<<endl;
	int rows = table->numRows();

	switch(gtype) {
	case GRAPH2D: {
		double x,y,xmin=0,xmax=0,ymin=0,ymax=0;
		Point *ptr = ((Graph2D *)graph)->Data();
		for (int i=0;i<rows;i++) {
			x=(table->text(i,0)).toDouble();
			y=(table->text(i,1)).toDouble();

			ptr[i].setPoint(x,y);
			LTableItem *item = (LTableItem *) table->item(i,0);
			if(item && item->Masked())
				ptr[i].setMasked();
		}

		mw->calculateRanges2D(ptr,rows,&xmin,&xmax,&ymin,&ymax);

		LRange range[2];
		range[0] = LRange(xmin,xmax);
		range[1] = LRange(ymin,ymax);
		((Graph2D *)graph)->setRange(range);

		kdDebug()<<"EditDialog :"<<endl;
		kdDebug()<<"new 2D Range :"<<endl;
		kdDebug()<<"		"<<xmin<<' '<<xmax<<endl;
		kdDebug()<<"		"<<ymin<<' '<<ymax<<endl;

		((Graph2D *)graph)->setNumber(rows);
		}; break;
	case GRAPH3D: {
		double x,y,z,xmin=0,xmax=0,ymin=0,ymax=0,zmin=0,zmax=0;
		Point3D *ptr = ((Graph3D *)graph)->Data();

		for (int i=0;i<rows;i++) {
			x=(table->text(i,0)).toDouble();
			y=(table->text(i,1)).toDouble();
			z=(table->text(i,2)).toDouble();

			ptr[i].setPoint(x,y,z);
			LTableItem *item = (LTableItem *) table->item(i,0);
			if(item && item->Masked())
				ptr[i].setMasked();
		}

		mw->calculateRanges3D(ptr,rows,&xmin,&xmax,&ymin,&ymax,&zmin,&zmax);

		LRange range[3];
		range[0] = LRange(xmin,xmax);
		range[1] = LRange(ymin,ymax);
		range[2] = LRange(zmin,zmax);

		kdDebug()<<"EditDialog :"<<endl;
		kdDebug()<<"3D Range :"<<endl;
		kdDebug()<<"		"<<xmin<<' '<<xmax<<endl;
		kdDebug()<<"		"<<ymin<<' '<<ymax<<endl;
		kdDebug()<<"		"<<zmin<<' '<<zmax<<endl;

		((Graph3D *)graph)->setRange(range);
		((Graph3D *)graph)->setNumber(rows);
		}; break;
	case GRAPHM : {
		double z, zmin=0,zmax=0;
		double *a = ((GraphM *)graph)->Data();

		for (int i=0;i<rows;i++) {
			for (int j=0;j<table->numCols();j++) {
				z=(table->text(i,j)).toDouble();
				if (i == 0) {
					zmin=zmax=z;
				}
				else {
					z<zmin?zmin=z:0;
					z>zmax?zmax=z:0;
				}
//				kdDebug()<<" EditDialog graphm : z = "<<z<<endl;

				a[j+i*(table->numCols())]=z;
			}
		}
		LRange range[1];
		range[0] = LRange(zmin,zmax);

		((GraphM *)graph)->setZRange(range);
		((GraphM *)graph)->setNumber(table->numCols(),rows);
		}; break;
	case GRAPH4D : {
		double x,y,z,t,xmin=0,xmax=0,ymin=0,ymax=0,zmin=0,zmax=0,tmin=0,tmax=0;
		Point4D *ptr = ((Graph4D *)graph)->Data();

		for (int i=0;i<rows;i++) {
			x=(table->text(i,0)).toDouble();
			y=(table->text(i,1)).toDouble();
			z=(table->text(i,2)).toDouble();
			t=(table->text(i,3)).toDouble();

			ptr[i].setPoint(x,y,z,t);
			LTableItem *item = (LTableItem *) table->item(i,0);
			if(item && item->Masked())
				ptr[i].setMasked();
		}

		mw->calculateRanges4D(ptr,rows,&xmin,&xmax,&ymin,&ymax,&zmin,&zmax,&tmin,&tmax);

		LRange range[4];
		range[0] = LRange(xmin,xmax);
		range[1] = LRange(ymin,ymax);
		range[2] = LRange(zmin,zmax);
		range[3] = LRange(tmin,tmax);

		kdDebug()<<"EditDialog :"<<endl;
		kdDebug()<<"4D Range :"<<endl;
		kdDebug()<<"		"<<xmin<<' '<<xmax<<endl;
		kdDebug()<<"		"<<ymin<<' '<<ymax<<endl;
		kdDebug()<<"		"<<zmin<<' '<<zmax<<endl;
		kdDebug()<<"		"<<tmin<<' '<<tmax<<endl;

		((Graph4D *)graph)->setRange(range);
		((Graph4D *)graph)->setNumber(rows);
		}; break;
	case GRAPHL : {
		// TODO
		}; break;
	default: break;
	}

	ld->updateList();
}

// build menu for context or "Spreadsheet"
void Spreadsheet::Menu(QPopupMenu *menu) {
	kdDebug()<<"Spreadsheet::Menu()"<<endl;

	plotmenu = new QPopupMenu(this);
	destmenu = new QPopupMenu(this);
	selmenu = new QPopupMenu(this);
	fillmenu = new QPopupMenu(this);
	normmenu = new QPopupMenu(this);
	editmenu = new QPopupMenu(this);
	convertmenu = new QPopupMenu(this);
	sortmenu = new QPopupMenu(this);
	maskmenu = new QPopupMenu(this);

	menu->clear();
	plotmenu->clear();
	destmenu->clear();
	selmenu->clear();
	fillmenu->clear();
	normmenu->clear();
	editmenu->clear();
	convertmenu->clear();
	sortmenu->clear();
	maskmenu->clear();
	kdDebug()<<"OK all cleared()"<<endl;

	QPixmap plot2dIcon = QPixmap(plot2d_xpm);
	QPixmap plot3dIcon = QPixmap(plot3d_xpm);
	QPixmap plotqwt3dIcon = QPixmap(plotqwt3d_xpm);
	QPixmap plotsurfaceIcon = QPixmap(plotsurface_xpm);
	QPixmap plotpieIcon = QPixmap(plotpie_xpm);
	QPixmap plotpolarIcon = QPixmap(plotpolar_xpm);
	QPixmap plotternaryIcon = QPixmap(plotternary_xpm);

	QPixmap spreadsheetIcon = QPixmap(spreadsheet_xpm);

	QPixmap selectIcon = KGlobal::iconLoader()->loadIcon("select", KIcon::Small);

	QPixmap cutIcon = KGlobal::iconLoader()->loadIcon("editcut", KIcon::Small);
	QPixmap copyIcon = KGlobal::iconLoader()->loadIcon("editcopy", KIcon::Small);
	QPixmap pasteIcon = KGlobal::iconLoader()->loadIcon("editpaste", KIcon::Small);
	QPixmap clearIcon = KGlobal::iconLoader()->loadIcon("editclear", KIcon::Small);

	QPixmap setIcon = KGlobal::iconLoader()->loadIcon("pagesetup", KIcon::Small);
	QPixmap fillIcon = KGlobal::iconLoader()->loadIcon("fill", KIcon::Small);
	QPixmap normIcon = KGlobal::iconLoader()->loadIcon("tool_normal", KIcon::Small);
	QPixmap convertIcon = KGlobal::iconLoader()->loadIcon("transform", KIcon::Small);
	QPixmap exportIcon = KGlobal::iconLoader()->loadIcon("fileexport", KIcon::Small);

	QPixmap deleteIcon = KGlobal::iconLoader()->loadIcon("editdelete", KIcon::Small);
	QPixmap addIcon = KGlobal::iconLoader()->loadIcon("edit_add", KIcon::Small);
	QPixmap sortIcon = KGlobal::iconLoader()->loadIcon("unsorted_list", KIcon::Small);
	QPixmap sortascIcon = KGlobal::iconLoader()->loadIcon("sort_incr", KIcon::Small);
	QPixmap sortdesIcon = KGlobal::iconLoader()->loadIcon("sort_decrease", KIcon::Small);
	QPixmap propertiesIcon = KGlobal::iconLoader()->loadIcon("site_properties", KIcon::Small);

	QPixmap maskIcon = QPixmap(plotmask_xpm);
	QPixmap columnstatIcon = QPixmap(columnstat_xpm);
	QPixmap rowstatIcon = QPixmap(rowstat_xpm);

	menu->insertItem(plot2dIcon, i18n( "Plot" ),plotmenu );
	plotmenu->insertItem(plot2dIcon, i18n( "2D Plot (XY)" ),this,SLOT(plot2DSimple()) );
	plotmenu->insertItem(plot2dIcon, i18n( "2D Plot (XYDY)" ),this,SLOT(plot3DXYDY()) );
	plotmenu->insertItem(plot2dIcon, i18n( "2D Plot (XYDXDY)" ),this,SLOT(plot4DXYDXDY()) );
	plotmenu->insertItem(plot2dIcon, i18n( "2D Plot (XYDYDY)" ),this,SLOT(plot4DXYDYDY()) );
	plotmenu->insertItem(plot2dIcon, i18n( "2D Plot (XY%X)" ),this,SLOT(plot3DXYPX()) );
	plotmenu->insertItem(plot2dIcon, i18n( "2D Plot (XY%Y)" ),this,SLOT(plot3DXYPY()) );
	plotmenu->insertItem(plot2dIcon, i18n( "2D Plot (LINE)" ),this,SLOT(plot2DLine()) );
	plotmenu->insertSeparator();
	plotmenu->insertItem(plot3dIcon, i18n( "3D Plot (XYZ)" ),this,SLOT(plot3DSimple()) );
	plotmenu->insertItem(plot3dIcon, i18n( "3D Plot (MATRIX)" ),this,SLOT(plotM3D()) );
	plotmenu->insertItem(plotqwt3dIcon, i18n( "3D QWT Plot (XYZ)" ),this,SLOT(plotQWT3D()) );
	plotmenu->insertItem(plotqwt3dIcon, i18n( "3D QWT Plot (MATRIX)" ),this,SLOT(plotMQWT3D()) );
	plotmenu->insertSeparator();
	plotmenu->insertItem(plotsurfaceIcon, i18n( "Surface Plot(XYZ)" ),this,SLOT(plot3DSurface()) );
	plotmenu->insertItem(plotsurfaceIcon, i18n( "Surface Plot(MATRIX)" ),this,SLOT(plotMSurface()) );
	plotmenu->insertSeparator();
	plotmenu->insertItem(plotpieIcon, i18n( "Pie Plot (XY)" ),this,SLOT(plot2DPie()) );
	plotmenu->insertItem(plotpieIcon, i18n( "Pie Plot (VL)" ),this,SLOT(plotLPie()) );
	plotmenu->insertItem(plotpolarIcon, i18n( "Polar Plot (XY)" ),this,SLOT(plot2DPolar()) );
	plotmenu->insertItem(plotternaryIcon, i18n( "Ternary Plot (XYZ)" ),this,SLOT(plot3DTernary()) );
	menu->insertItem( selectIcon, i18n( "Set title" ),this,SLOT(setTitle()) );
	menu->insertItem( selectIcon, i18n( "Select destination" ),this,SLOT(selectDestination()) );
	menu->insertItem( selectIcon, i18n( "Edit Notes" ),this,SLOT(setNotes()) );
	menu->insertSeparator();
	menu->insertItem( cutIcon, i18n( "Cut" ), this, SLOT(cutSelection()), CTRL+Key_X );
	menu->insertItem( copyIcon, i18n( "Copy" ), this, SLOT(copySelection()), CTRL+Key_C );
	menu->insertItem( pasteIcon, i18n( "Paste" ), this, SLOT(pasteSelection()), CTRL+Key_V );
	menu->insertItem( clearIcon, i18n( "Clear" ), this, SLOT(clearSelection()) );
	menu->insertItem( selectIcon, i18n( "Selection" ), selmenu );
	selmenu->insertItem(selectIcon, i18n("Select all"),this,SLOT(selectAll()), CTRL+Key_A);
	selmenu->insertItem(selectIcon, i18n("Deselect all"),this,SLOT(selectNone()));
	selmenu->insertItem(selectIcon, i18n("Invert selection"),this,SLOT(selectInvert()));
	menu->insertSeparator();
	menu->insertItem( setIcon, i18n( "Set column values" ),this, SLOT(setValues()),CTRL+Key_Semicolon );
	menu->insertItem( fillIcon, i18n( "Fill with" ),fillmenu );
	fillmenu->insertItem( i18n( "Row number" ),this,SLOT(fillRowNumber()) );
	fillmenu->insertItem( i18n( "Random values" ),this,SLOT(fillRandom()) );
	menu->insertItem( normIcon, i18n( "Normalize" ),normmenu );
	normmenu->insertItem( i18n( "Sum=1" ),this,SLOT(normSum()) );
	normmenu->insertItem( i18n( "Max=1" ),this,SLOT(normMax()) );
	menu->insertItem( copyIcon, i18n( "Edit with editor" ),editmenu );
	editmenu->insertItem( i18n( "vi" ),this,SLOT(editVI()) );
	editmenu->insertItem( i18n( "kvim" ),this,SLOT(editKVIM()) );
	editmenu->insertItem( i18n( "gvim" ),this,SLOT(editGVIM()) );
//	this doesn't work
//	editmenu->insertItem( i18n( "kate" ),this,SLOT(editKATE()) );
	editmenu->insertItem( i18n( "kwrite" ),this,SLOT(editKWRITE()) );
	editmenu->insertItem( i18n( "Emacs" ),this,SLOT(editEMACS()) );
	editmenu->insertItem( i18n( "Xemacs" ),this,SLOT(editXEMACS()) );
	editmenu->insertItem( i18n( "kword" ),this,SLOT(editKWORD()) );
	editmenu->insertItem( i18n( "Starwrite" ),this,SLOT(editSOFFICE()) );
	menu->insertSeparator();
	menu->insertItem( convertIcon, i18n( "Convert" ),convertmenu );
	convertmenu->insertItem( i18n( "Replace ',' with '.'" ),this,SLOT(convertComma()) );
	convertmenu->insertItem( i18n( "Convert FORTRAN data" ),this,SLOT(convertFORTRAN()) );
	convertmenu->insertItem( i18n( "Transpose Matrix" ),this,SLOT(transposeMatrix()) );
	convertmenu->insertItem( i18n( "Matrix -> XYZ" ), this,SLOT(convertMatrixtoXYZ()) );
	convertmenu->insertItem( i18n( "XYZ -> Matrix" ), this, SLOT(convertXYZtoMatrix()) );
	convertmenu->insertItem( i18n("Column -> Matrix"), this, SLOT(convertColumntoMatrix()) );
	menu->insertSeparator();
	menu->insertItem(exportIcon, i18n("Export data"),this, SLOT(exportData()));
	menu->insertSeparator();
	menu->insertItem( addIcon, i18n("Add Columns"), this, SLOT(addColumn()) );
	menu->insertItem(spreadsheetIcon, i18n("New S&preadsheet"),mw,SLOT(newSpreadsheet()),CTRL+Key_Equal);
	menu->insertItem( deleteIcon, i18n( "Delete selected Columns" ), this, SLOT(deleteColumns()) );
	menu->insertItem( deleteIcon, i18n( "Delete selected Rows" ), this, SLOT(deleteRows()) );
	menu->insertItem( maskIcon, i18n( "Masking" ), maskmenu );
	maskmenu->insertItem( maskIcon, i18n( "Mask/Unmask selection" ),this,SLOT(maskSelection()) );
	maskmenu->insertItem( maskIcon, i18n( "Toggle Masking" ),this,SLOT(toggleMask()) );
	maskmenu->insertItem( maskIcon, i18n( "Unmask all" ),this,SLOT(unMask()) );
	maskmenu->insertItem( maskIcon, i18n( "Mask every n-th row" ),this,SLOT(maskNthRow()) );
	maskmenu->insertItem( maskIcon, i18n( "Mask all but n-th row" ),this,SLOT(maskFirstRow()) );
	menu->insertSeparator();
	menu->insertItem( sortIcon, i18n( "Sort" ),sortmenu );
	sortmenu->insertItem( sortascIcon, i18n( "Ascending" ),this,SLOT(sortAscending()) );
	sortmenu->insertItem( sortdesIcon, i18n( "Descending" ),this,SLOT(sortDescending()) );
	menu->insertItem( columnstatIcon, i18n( "Statistics on Columns" ),this,SLOT(Statistics()) );
	menu->insertItem( rowstatIcon, i18n( "Statistics on Rows" ),this,SLOT(rowStatistics()) );
	menu->insertSeparator();
	menu->insertItem(propertiesIcon, i18n( "Properties" ),this,SLOT(setProperties()) );
}

// used to update spreadsheet after UNDO
void Spreadsheet::update() {
	setCaption(title);
}

void Spreadsheet::contextMenuEvent(QContextMenuEvent *) {
	kdDebug()<<"Spreadsheet::contextMenuEvent()"<<endl;

	//TODO : if over header : select current column
/*	if ( object == (QObject *)table->horizontalHeader()) {
		const QMouseEvent *me = (const QMouseEvent *)e;
		int col=table->horizontalHeader()->sectionAt (me->pos().x());
		kdDebug()<<"	HEADER : selection col "<<col<<endl;
		if(!table->isColumnSelected(col))
			selectColumns(col);
	}*/

	// crashes here after UNDO !
	QPopupMenu *menu = new QPopupMenu(this);
//	kdDebug()<<"OK"<<endl;
	Menu(menu);
//	kdDebug()<<"OK"<<endl;

	menu->exec(QCursor::pos());
}

void Spreadsheet::openXML(QDomNode node) {
	while(!node.isNull()) {
		QDomElement e = node.toElement();
//		kdDebug()<<"SS TAG = "<<e.tagName()<<endl;
//		kdDebug()<<"SS TEXT = "<<e.text()<<endl;

		if(e.tagName() == "Position") {
			int px = e.attribute("x").toInt();
			int py = e.attribute("y").toInt();
			parentWidget()->move(QPoint(px,py));
			if(px<0 || py<0) {	// fullscreen
				mw->getWorkspace()->cascade();
				showMaximized();
			}
		}
		else if(e.tagName() == "Size") {
			table->setNumRows(e.attribute("rows").toInt());
			table->setNumCols(e.attribute("cols").toInt());
		}
		else if(e.tagName() == "Title")
			title = e.text();
		else if(e.tagName() == "Notes")
			notes = e.text();
		else if(e.tagName() == "Datafile")
			datafile = e.text();
		else if(e.tagName() == "Column")
			table->horizontalHeader()->setLabel(e.attribute("nr").toInt(),e.text());
		else if(e.tagName() == "Cell") {
			LTableItem *item = new LTableItem( table, QTableItem::OnTyping, e.text());
			item->setMasked(e.attribute("masked").toInt());
			table->setItem(e.attribute("row").toInt(), e.attribute("col").toInt(), item);
		}

		node = node.nextSibling();
	}
}

QDomElement Spreadsheet::saveXML(QDomDocument doc) {
	QDomElement sstag = doc.createElement( "Spreadsheet" );
	QDomElement tag = doc.createElement( "Position" );
	tag.setAttribute("x",QString::number(parentWidget()->pos().x()));
	tag.setAttribute("y",QString::number(parentWidget()->pos().y()));
	sstag.appendChild( tag );
	tag = doc.createElement( "Size" );
	tag.setAttribute("rows",QString::number(table->numRows()));
	tag.setAttribute("cols",QString::number(table->numCols()));
	sstag.appendChild( tag );
	tag = doc.createElement( "Title" );
	sstag.appendChild( tag );
	QDomText t = doc.createTextNode( title );
	tag.appendChild( t );
	tag = doc.createElement( "Notes" );
	sstag.appendChild( tag );
	t = doc.createTextNode( notes );
	tag.appendChild( t );
	tag = doc.createElement( "Datafile" );
	sstag.appendChild( tag );
	t = doc.createTextNode( datafile );
 	tag.appendChild( t );

	// header
	for(int i=0;i<table->numCols();i++) {
		tag = doc.createElement( "Column" );
		tag.setAttribute("nr",QString::number(i));
		sstag.appendChild( tag );
  		t = doc.createTextNode( table->horizontalHeader()->label(i) );
 		tag.appendChild( t );
	}

	// data
	for(int i=0;i<table->numRows();i++) {
		for(int j=0;j<table->numCols();j++) {
			if(!table->text(i,j).isEmpty()) {	// only filled cells
				tag = doc.createElement( "Cell" );
				tag.setAttribute("row",QString::number(i));
				tag.setAttribute("col",QString::number(j));
				tag.setAttribute("masked",QString::number(((LTableItem *)table->item(i,j))->Masked()));
				sstag.appendChild( tag );
  				t = doc.createTextNode( table->text(i,j) );
 				tag.appendChild( t );
			}
		}
	}

	return sstag;
}

void Spreadsheet::save(QTextStream *t) {
	kdDebug()<<"Spreadsheet::save()"<<endl;
	if(table == 0) return;

	*t<<title<<endl;
	*t<<table->numCols()<<' '<<table->numRows()<<endl;
	*t<<datafile<<endl;

	// dump header
	for(int i=0;i<table->numCols();i++)
		*t<<table->horizontalHeader()->label(i)<<endl;

	// dump data
	for(int i=0;i<table->numRows();i++) {
		// dump masking of first item
		LTableItem *item = (LTableItem *) table->item(i,0);
		*t<<"\""<<item->Masked()<<"\" ";

		for(int j=0;j<table->numCols();j++) {
			*t<<"\""<<table->text(i,j)<<"\" ";
		}
		*t<<endl;
	}
	kdDebug()<<"Spreadsheet::save() DONE"<<endl;
}

void Spreadsheet::open(QTextStream *t, int version) {
	kdDebug()<<"Spreadsheet::open()"<<endl;
	if(version>20) {
		t->readLine();
		title = t->readLine();
		kdDebug()<<"TITLE = "<<title<<endl;
		setCaption(title);
	}
	int nx,ny;
	*t>>nx>>ny;
	table->setNumCols(nx);
	table->setNumRows(ny);
	kdDebug()<<"NX="<<nx<<" NY="<<ny<<endl;

	if(version > 22) {
		// read datafile
		datafile = t->readLine();
	}

	// read header
	QString tmp;
	t->readLine();
	for(int i=0;i<table->numCols();i++) {
		tmp = t->readLine();
		// kdDebug()<<"Label "<<i<<'='<<tmp<<endl;
		table->horizontalHeader()->setLabel(i,tmp);
	}

	// read data
	for(int i=0;i<table->numRows();i++) {
		tmp = t->readLine();
		tmp = tmp.simplifyWhiteSpace();
		QStringList oneline;
		if(tmp.contains('"'))
			oneline = QStringList::split("\" ",tmp);	// new style
		else
			oneline = QStringList::split(" ",tmp);		// old style (only 1.4.0.pre/rc)
		int j=0, masked=false;
		for ( QStringList::Iterator it = oneline.begin(); it != oneline.end(); ++it ) {
				QString entry = *it;
				entry.remove('"');

				// read masking flag
				if(version > 21 && it == oneline.begin()) {
					masked = entry.toInt();
					continue;
				}

				LTableItem *item = new LTableItem( table, QTableItem::OnTyping,entry);
				item->setMasked(masked);
				table->setItem(i, j++, item);
		}
	}

	kdDebug()<<"Spreadsheet::open() DONE"<<endl;
}

//! return format index for column col
int Spreadsheet::formatItem(int col) {
	for(int i=0;i<NR_FORMATS;i++)
		if(columnFormat(col) == i18n(formatList[i]))
			return i;
	return -1;
}

QStringList Spreadsheet::columnInfo(int col) {
	QStringList list;

	list += columnTitle(col);
	list += columnType(col);
	list += columnFormat(col);

	return list;
}

// return name of column col
QString Spreadsheet::columnTitle(int col) {
	QString label = table->horizontalHeader()->label(col);
	label.remove(QRegExp(" \\{.+\\]"));

	return label;
}

void Spreadsheet::setColumnTitle(int col, QString name) {
	if(col==0)
		table->horizontalHeader()->setLabel(col, name+' '+i18n("{double}")+" [X]");
	else
		table->horizontalHeader()->setLabel(col, name+' '+i18n("{double}")+" [Y]");
}

QString Spreadsheet::columnType(int col) {
	QString header = table->horizontalHeader()->label(col);
	int pos1 = header.find(QRegExp("\\["));
	int pos2 = header.find(QRegExp("\\]"));
	return header.mid(pos1,pos2-pos1+1);
}

void Spreadsheet::setColumnType(int col, QString type) {
	kdDebug()<<"setColumnType() : col="<<col<<" type = "<<type<<endl;
	QString label = table->horizontalHeader()->label(col);
	label.replace(QRegExp(" \\[.+\\]"),QString(" ["+type+"]"));
	table->horizontalHeader()->setLabel(col, label);
}

QString Spreadsheet::columnFormat(int col) {
	QString header = table->horizontalHeader()->label(col);
	int pos1 = header.find(QRegExp("\\{"));
	int pos2 = header.find(QRegExp("\\}"));
	return header.mid(pos1,pos2-pos1+1);
}

void Spreadsheet::setColumnFormat(int col, QString format) {
	kdDebug()<<"setColumnFormat() : col="<<col<<" type = "<<format<<endl;
	QString label = table->horizontalHeader()->label(col);
	label.replace(QRegExp(" \\{.+\\}"),QString(" {"+format+"}"));
	table->horizontalHeader()->setLabel(col, label);
}

// set default column names
void Spreadsheet::setColumnNames(int nrcols) {
	for(int i=0;i<nrcols;i++) {
		if(i < 26)
			table->horizontalHeader()->setLabel( i, QChar(i+65)+' '+i18n("{double}")+" [Y]");
		else
			table->horizontalHeader()->setLabel( i,
		QString(QChar(65+i/26-1)) + QString(QChar(65+i%26)) + ' ' + i18n("{double}")+" [Y]");
	}
}

//! make 2d plot with x-y-dy
void Spreadsheet::plot3DXYDY(Input2D mode) {
	kdDebug()<<"Spreadsheet::plot3DXYDY("<<mode<<")"<<endl;

	// if less than 3 columns selected -> select all columns
	int selected_cols=0;
	for (int s=0;s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);
#if QT_VERSION > 0x030102
		selected_cols+=ts.numCols();
#else
		selected_cols+=ts.rightCol()-ts.leftCol();
#endif
	}
	if(selected_cols<2)
		for (int i=0;i<table->numCols();i++)
			table->selectColumn(i);

	// find x,y,dy column
	int indexx=-1,indexy=-1,indexz=-1;
	for(int i=table->numCols()-1;i>=0;i--) {
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[X]")>0)
			indexx=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[Y]")>0)
			indexy=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[DY]")>0)
			indexz=i;
	}

	Point3D *ptr = new Point3D[table->numRows()];
	// read x and y values
	double x, y, z, xmin=0,xmax=1,ymin=0,ymax=1, zmin=0, zmax=1;
	int numrows=0;

	// ask for percentage
	double percent=0.0;
	if(mode == I2DXYPX || mode == I2DXYPY) {
		bool ok;
		percent = QInputDialog::getDouble(i18n("2D error plot"),i18n("Percentage to use"),10,1,INF,1,&ok);
		if(!ok)
			return;
		kdDebug()<<"	percentage "<<percent<<endl;
	}

	// only selected rows
	QTableSelection ts = table->selection(table->currentSelection());
//	kdDebug()<<"NUM SEL = "<<table->numSelections()<<endl;
	int start=0, end=table->numRows();
	if(ts.bottomRow()>ts.topRow()) {
		start = ts.topRow();
		end =  ts.bottomRow();
	}
	kdDebug()<<"reading ROWS = "<<start<<' '<<end<<endl;
	for (int row = start; row < end + 1; row++) {
		if(indexx == -1)
			x = (double) row + 1;
		else
			x = mw->formatLabel(table->text(row,indexx),formatItem(indexx));
		y = mw->formatLabel(table->text(row,indexy),formatItem(indexy));
		if(mode == I2DXYPX)
			z = fabs(percent/100.0*x);
		else if(mode == I2DXYPY)
			z = fabs(percent/100.0*y);
		else
			z = mw->formatLabel(table->text(row,indexz),formatItem(indexz));
		//kdDebug()<<"x/y ="<<x<<' '<<y<<endl;

		// skip empty rows
		if (table->text(row,indexy).isEmpty())
			continue;

		bool nanvalue=false;
		if(!finite(x)) { x=0; nanvalue=true; }
		if(!finite(y)) { y=0; nanvalue=true; }
		if(!finite(z)) { z=0; nanvalue=true; }

		if(row==start) {
			xmin=xmax=x;
			ymin=ymax=y;
			zmin=zmax=z;
		}

		x<xmin?xmin=x:0;
		x>xmax?xmax=x:0;
		y<ymin?ymin=y:0;
		y>ymax?ymax=y:0;
		z<zmin?zmin=z:0;
		z>zmax?zmax=z:0;

		ptr[numrows].setPoint(x,y,z);
		// mask NaN values
		if(nanvalue)
			ptr[numrows].setMasked();

		LTableItem *item = (LTableItem *) table->item(row,indexy);
		if(item && item->Masked())
			ptr[numrows].setMasked();
		numrows++;
	};

	// make new Graph3D
	LRange range[3];
	range[0] = LRange(xmin,xmax);
	range[1] = LRange(ymin,ymax);
	range[2] = LRange(zmin,zmax);
	Style *style = mw->defaultStyle();
	Symbol *symbol = mw->defaultSymbol();

	Graph3D *g = new Graph3D(caption(),columnTitle(indexy),range,SSPREADSHEET,P2D,style,symbol,ptr,numrows,1);

	mw->addGraph3D(g,destination,P2D);

	kdDebug()<<"Spreadsheet::plot3DXYDY() DONE"<<endl;
}

//! make 2d plot with x-y-dx-dy
void Spreadsheet::plot4DXYDXDY() {
	kdDebug()<<"Spreadsheet::plot4DXYDXDY()"<<endl;

	// if less than 4 columns selected -> select all columns
	int selected_cols=0;
	for (int s=0;s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);
#if QT_VERSION > 0x030102
		selected_cols+=ts.numCols();
#else
		selected_cols+=ts.rightCol()-ts.leftCol();
#endif
	}
	if(selected_cols<3)
		for (int i=0;i<table->numCols();i++)
			table->selectColumn(i);

	// find x,y,dy column
	int indexx=-1,indexy=-1,indexz=-1, indext=-1;
	for(int i=table->numCols()-1;i>=0;i--) {
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[X]")>0)
			indexx=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[Y]")>0)
			indexy=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[DX]")>0)
			indexz=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[DY]")>0)
			indext=i;
	}

	Point4D *ptr = new Point4D[table->numRows()];
	// read x and y values
	double x, y, z, t, xmin=0,xmax=1,ymin=0,ymax=1, zmin=0, zmax=1, tmin=0,tmax=1;
	int numrows=0;

	// only selected rows
	QTableSelection ts = table->selection(table->currentSelection());
//	kdDebug()<<"NUM SEL = "<<table->numSelections()<<endl;
	int start=0, end=table->numRows();
	if(ts.bottomRow()>ts.topRow()) {
		start = ts.topRow();
		end =  ts.bottomRow();
	}
	kdDebug()<<"reading ROWS = "<<start<<' '<<end<<endl;
	for (int row = start; row < end + 1; row++) {
		if(indexx == -1)
			x = (double) row + 1;
		else
			x = mw->formatLabel(table->text(row,indexx),formatItem(indexx));
		y = mw->formatLabel(table->text(row,indexy),formatItem(indexy));
		z = mw->formatLabel(table->text(row,indexz),formatItem(indexz));
		t = mw->formatLabel(table->text(row,indext),formatItem(indext));
		//kdDebug()<<"x/y ="<<x<<' '<<y<<endl;

		// skip empty rows
		if (table->text(row,indexy).isEmpty())
			continue;

		bool nanvalue=false;
		if(!finite(x)) { x=0; nanvalue=true; }
		if(!finite(y)) { y=0; nanvalue=true; }
		if(!finite(z)) { z=0; nanvalue=true; }
		if(!finite(t)) { t=0; nanvalue=true; }

		if(row==start) {
			xmin=xmax=x;
			ymin=ymax=y;
			zmin=zmax=z;
			tmin=tmax=t;
		}

		x<xmin?xmin=x:0;
		x>xmax?xmax=x:0;
		y<ymin?ymin=y:0;
		y>ymax?ymax=y:0;
		z<zmin?zmin=z:0;
		z>zmax?zmax=z:0;
		t<tmin?tmin=t:0;
		t>tmax?tmax=t:0;

		ptr[numrows].setPoint(x,y,z,t);
		if(nanvalue)
			ptr[numrows].setMasked();

		LTableItem *item = (LTableItem *) table->item(row,indexy);
		if(item && item->Masked())
			ptr[numrows].setMasked();
		numrows++;
	};

	// make new Graph4D
	LRange range[4];
	range[0] = LRange(xmin,xmax);
	range[1] = LRange(ymin,ymax);
	range[2] = LRange(zmin,zmax);
	range[3] = LRange(tmin,tmax);
	Style *style = mw->defaultStyle();
	Symbol *symbol = mw->defaultSymbol();

	Graph4D *g = new Graph4D(caption(),columnTitle(indexy),range,SSPREADSHEET,P2D,style,symbol,ptr,numrows,false);

	mw->addGraph4D(g,destination);
}

//! make 2d plot with x-y-dy1-dy2
void Spreadsheet::plot4DXYDYDY() {
	kdDebug()<<"Spreadsheet::plot4DXYDYDY()"<<endl;

	// if less than 4 columns selected -> select all columns
	int selected_cols=0;
	for (int s=0;s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);
#if QT_VERSION > 0x030102
		selected_cols+=ts.numCols();
#else
		selected_cols+=ts.rightCol()-ts.leftCol();
#endif
	}
	if(selected_cols<3)
		for (int i=0;i<table->numCols();i++)
			table->selectColumn(i);

	// find x,y,dy column
	int indexx=-1,indexy=-1,indexz=-1, indext=-1;
	for(int i=table->numCols()-1;i>=0;i--) {
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[X]")>0)
			indexx=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[Y]")>0)
			indexy=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[DY]")>0)
			indexz=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[DY2]")>0)
			indext=i;
	}

	Point4D *ptr = new Point4D[table->numRows()];
	// read x and y values
	double x, y, z, t, xmin=0,xmax=1,ymin=0,ymax=1, zmin=0, zmax=1, tmin=0,tmax=1;
	int numrows=0;

	// only selected rows
	QTableSelection ts = table->selection(table->currentSelection());
//	kdDebug()<<"NUM SEL = "<<table->numSelections()<<endl;
	int start=0, end=table->numRows();
	if(ts.bottomRow()>ts.topRow()) {
		start = ts.topRow();
		end =  ts.bottomRow();
	}
	kdDebug()<<"reading ROWS = "<<start<<' '<<end<<endl;
	for (int row=start;row<end+1;row++) {
		if(indexx==-1)
			x = (double) row + 1;
		else
			x = mw->formatLabel(table->text(row,indexx),formatItem(indexx));
		y = mw->formatLabel(table->text(row,indexy),formatItem(indexy));
		z = mw->formatLabel(table->text(row,indexz),formatItem(indexz));
		t = mw->formatLabel(table->text(row,indext),formatItem(indext));
		//kdDebug()<<"x/y ="<<x<<' '<<y<<endl;

		// skip empty rows
		if (table->text(row,indexy).isEmpty())
			continue;

		bool nanvalue=false;
		if(!finite(x)) { x=0; nanvalue=true; }
		if(!finite(y)) { y=0; nanvalue=true; }
		if(!finite(z)) { z=0; nanvalue=true; }
		if(!finite(t)) { t=0; nanvalue=true; }

		if(row==start) {
			xmin=xmax=x;
			ymin=ymax=y;
			zmin=zmax=z;
			tmin=tmax=t;
		}

		x<xmin?xmin=x:0;
		x>xmax?xmax=x:0;
		y<ymin?ymin=y:0;
		y>ymax?ymax=y:0;
		z<zmin?zmin=z:0;
		z>zmax?zmax=z:0;
		t<tmin?tmin=t:0;
		t>tmax?tmax=t:0;

		ptr[numrows].setPoint(x,y,z,t);
		if(nanvalue)
			ptr[numrows].setMasked();

		LTableItem *item = (LTableItem *) table->item(row,indexy);
		if(item && item->Masked())
			ptr[numrows].setMasked();
		numrows++;
	};

	// make new Graph4D
	LRange range[4];
	range[0] = LRange(xmin,xmax);
	range[1] = LRange(ymin,ymax);
	range[2] = LRange(zmin,zmax);
	range[3] = LRange(tmin,tmax);
	Style *style = mw->defaultStyle();
	Symbol *symbol = mw->defaultSymbol();

	Graph4D *g = new Graph4D(caption(),columnTitle(indexy),range,SSPREADSHEET,P2D,style,symbol,ptr,numrows,true);

	mw->addGraph4D(g,destination);
}

void Spreadsheet::plot2D(PType type, bool line) {
	kdDebug()<<"Spreadsheet::plot2D() : "<<type<<endl;

	// if less than 2 columns selected -> select all columns
	int selected_cols=0;
	for (int s=0;s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);
#if QT_VERSION > 0x030102
		selected_cols+=ts.numCols();
#else
		selected_cols+=ts.rightCol()-ts.leftCol();
#endif
	}
	if(selected_cols<1 && !line)
		for (int i=0;i<table->numCols();i++)
			table->selectColumn(i);

	// find x column
	int indexx=-1;
	for(int i=table->numCols()-1;i>=0;i--) {
		// use only selected columns
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[X]")>0)
			indexx=i;
	}

	if(line) {
		// read x and y values
		double x, y, xmin=0,xmax=1,ymin=0,ymax=1;

		for (int s=0;s<table->numSelections();s++) {
			QTableSelection ts = table->selection(s);
			kdDebug()<<" rows "<<ts.topRow()<<" to row "<<ts.bottomRow()<<endl;

			for (int row=ts.topRow();row<=ts.bottomRow();row++) {
				Point *ptr = new Point[table->numCols()];
				for (int col=0;col<table->numCols();col++) {
					x =(double) col+1;
					// formatItem should be double for all columns. datetime ?
					y = mw->formatLabel(table->text(row,col),formatItem(col));
					kdDebug()<<"col/row : " <<col<<'/'<<row<<endl;

					bool nanvalue=false;
					if(!finite(x)) { x=0; nanvalue=true; }
					if(!finite(y)) { y=0; nanvalue=true; }

					if(col==0) {
						xmin=xmax=x;
						ymin=ymax=y;
					}

					x<xmin?xmin=x:0;
					x>xmax?xmax=x:0;
					y<ymin?ymin=y:0;
					y>ymax?ymax=y:0;

					ptr[col].setPoint(x,y);
					if(nanvalue)
						ptr[col].setMasked();

					LTableItem *item = (LTableItem *) table->item(row,col+1);
					if(item && item->Masked())
						ptr[col].setMasked();
				}

				// make new Graph2D
				LRange range[2];
				range[0] = LRange(xmin,xmax);
				range[1] = LRange(ymin,ymax);
				Style *style = mw->defaultStyle();
				Symbol *symbol = mw->defaultSymbol();

				Graph2D *g = new Graph2D(caption(),i18n("row")+' '+QString::number(row+1),range,
					SSPREADSHEET,type,style,symbol,ptr,table->numCols());

				// use the same destination for every row
				destination = mw->addGraph2D(g,destination,type);
				kdDebug()<<"Setting destination to "<<destination<<endl;
			}
		}
	}
	else {	// no line data
		for(int i=0;i<table->numCols();i++) {
			// use only selected y cols
			if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[Y]")>0) {
				Point *ptr = new Point[table->numRows()];
				// read x and y values
				double x, y, xmin=0,xmax=1,ymin=0,ymax=1;
				int numrows=0;

				// only selected rows
				QTableSelection ts = table->selection(table->currentSelection());
//				kdDebug()<<"NUM SEL = "<<table->numSelections()<<endl;
				int start=0, end=table->numRows();
				if(ts.bottomRow()>ts.topRow()) {
					start = ts.topRow();
					end =  ts.bottomRow();
				}
				kdDebug()<<"reading ROWS = "<<start<<' '<<end<<endl;
				for (int row = start; row < end+1; row++) {
					if(indexx == -1)
						x =(double) row + 1;
					else
						x = mw->formatLabel(table->text(row,indexx),formatItem(indexx));

					// skip empty rows
					if (table->text(row,i).isEmpty())
						continue;

					y = mw->formatLabel(table->text(row,i),formatItem(i));

					bool nanvalue=false;
					if(!finite(x)) { x=0; nanvalue=true; }
					if(!finite(y)) { y=0; nanvalue=true; }

					if(row==start) {
						xmin=xmax=x;
						ymin=ymax=y;
					}

					x<xmin?xmin=x:0;
					x>xmax?xmax=x:0;
					y<ymin?ymin=y:0;
					y>ymax?ymax=y:0;

					ptr[numrows].setPoint(x,y);
					if(nanvalue)
						ptr[numrows].setMasked();

					LTableItem *item = (LTableItem *) table->item(row,i);
					if(item && item->Masked())
						ptr[numrows].setMasked();
					numrows++;
				}
				kdDebug()<<"xmin/xmax = "<<(int)xmin<<' '<<(int)xmax<<endl;

				// make new Graph2D
				LRange range[2];
				range[0] = LRange(xmin,xmax);
				range[1] = LRange(ymin,ymax);
				Style *style = mw->defaultStyle();
				Symbol *symbol = mw->defaultSymbol();

				Graph2D *g = new Graph2D(caption(),columnTitle(i),range,SSPREADSHEET,
					type,style,symbol,ptr,numrows);

				// use the same destination for every row
				destination = mw->addGraph2D(g,destination,type);
				kdDebug()<<"Setting destination to "<<destination<<endl;
			}
		}
	}
}

void Spreadsheet::plot3D(PType type) {
	kdDebug()<<"Spreadsheet::plot3D() : "<<type<<endl;

	// if less than 3 columns selected -> select all columns
	int selected_cols=0;
	for (int s=0;s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);
#if QT_VERSION > 0x030102
		selected_cols+=ts.numCols();
#else
		selected_cols+=ts.rightCol()-ts.leftCol();
#endif
	}
	if(selected_cols<3)
		for (int i=0;i<table->numCols();i++)
			table->selectColumn(i);

	// find x,y,z column
	int indexx=-1, indexy=-1, indexz=-1;
	for(int i=table->numCols()-1;i>=0;i--) {
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[X]")>0)
			indexx=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[Y]")>0)
			indexy=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[Z]")>0)
			indexz=i;
	}

	// use all z cols
	for(int i=0;i<table->numCols();i++) {
		// if no Z column : use the last one
		if((table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[Z]")>0) ||
			(indexz==-1 && i==table->numCols()-1)) {
			Point3D *tmpptr = new Point3D[table->numRows()];
			// read x and y values
			double xmin=0,xmax=1,ymin=0,ymax=1,zmin=0,zmax=1;
			int numrows=0;

			// only selected rows
			QTableSelection ts = table->selection(table->currentSelection());
//			kdDebug()<<"NUM SEL = "<<table->numSelections()<<endl;
			int start=0, end=table->numRows();
			if(ts.bottomRow()>ts.topRow()) {
				start = ts.topRow();
				end =  ts.bottomRow();
			}
			kdDebug()<<"reading ROWS = "<<start<<' '<<end<<endl;
			for (int row = start; row < end + 1; row++) {
				double x = mw->formatLabel(table->text(row,indexx),formatItem(indexx));
				double y = mw->formatLabel(table->text(row,indexy),formatItem(indexy));
				double z = mw->formatLabel(table->text(row,i),formatItem(i));
				// kdDebug()<<"x/y/z ="<<x<<' '<<y<<' '<<z<<endl;

				// skip empty rows
				if (table->text(row,i).isEmpty())
					continue;

				bool nanvalue=false;
				if(!finite(x)) { x=0; nanvalue=true; }
				if(!finite(y)) { y=0; nanvalue=true; }
				if(!finite(z)) { z=0; nanvalue=true; }

				if (row == start) {
					kdDebug()<<"TYPE = "<<type<<endl;
					if (type == PTERNARY) {
						xmin=0;
						xmax=x+y+z;
						kdDebug()<<"TERNARY MAX = "<<xmax<<endl;
					}
					else
						xmin=xmax=x;
					ymin=ymax=y;
					zmin=zmax=z;
				}
				else {
					if (type != PTERNARY) {
						x<xmin?xmin=x:0;
						x>xmax?xmax=x:0;
					}
					y<ymin?ymin=y:0;
					y>ymax?ymax=y:0;
					z<zmin?zmin=z:0;
					z>zmax?zmax=z:0;
				}

				tmpptr[numrows].setPoint(x,y,z);
				if(nanvalue)
					tmpptr[numrows].setMasked();

				LTableItem *item = (LTableItem *) table->item(row,indexy);
				if(item && item->Masked())
					tmpptr[numrows].setMasked();
				numrows++;
			};

			// make new Graph3D
			LRange range[3];
			range[0] = LRange(xmin,xmax);
			range[1] = LRange(ymin,ymax);
			range[2] = LRange(zmin,zmax);
			Style *style = mw->defaultStyle();
			Symbol *symbol = mw->defaultSymbol();
			kdDebug()<<"X RANGE : "<<xmin<<' '<<xmax<<endl;
			kdDebug()<<"Y RANGE : "<<ymin<<' '<<ymax<<endl;
			kdDebug()<<"Z RANGE : "<<zmin<<' '<<zmax<<endl;

			int cols=1, maxcols=0;
			Point3D *ptr=0;
			bool swap=false;

			// TEST : P3D
			if(type == PTERNARY || type == PQWT3D || type == P3D) {
				maxcols=1;
				ptr=tmpptr;
			}
			else if (type == PSURFACE) {	// convert to matrix form
				// calculate dimension
				int rows=0;
				for(int i=0;i<table->numRows();i++) {
//					kdDebug()<<"ROW "<<i+1<<endl;
					if(i == table->numRows()-1) {
						rows++;
						maxcols<cols?maxcols=cols:0;
//						kdDebug()<<"	ROW "<<rows<<" has "<<cols<<" columns"<<endl;
					}
					else if(tmpptr[i].X()>tmpptr[i+1].X() || tmpptr[i].Y()>tmpptr[i+1].Y() ) {
						if(tmpptr[i].Y()>tmpptr[i+1].Y())		// y is running
							swap=true;
						rows++;
						maxcols<cols?maxcols=cols:0;
//						kdDebug()<<"	ROW "<<rows<<" has "<<cols<<" columns"<<endl;
						cols=1;
					}
					else
						cols++;
				}

				// convert data and create new ptr
//				kdDebug()<<"	rows/cols = "<<rows<<'/'<<maxcols<<endl;
				ptr = new Point3D[rows*maxcols];
				for(int i=0;i<rows;i++)
					for(int j=0;j<maxcols;j++)
						ptr[i*maxcols+j].setPoint(xmin+j*(xmax-xmin)/(maxcols-1),ymin+i*(ymax-ymin)/(rows-1),0);

				int index=0;
				for(int i=0;i<rows;i++) {
					for(int j=0;j<maxcols;j++) {
//						kdDebug()<<tmpptr[index].X()<<' '<<tmpptr[index+1].X()<<endl;
//						kdDebug()<<"SET index "<<index<<" to i*maxcols+j="<<i*maxcols+j<<" : "<<tmpptr[index].X()<<endl;
						ptr[i*maxcols+j].setPoint(tmpptr[index].X(),tmpptr[index].Y(),tmpptr[index].Z());
						ptr[i*maxcols+j].setMasked(ptr[index].Masked());
						if(index == table->numRows()-1 ||
							tmpptr[index].X()>tmpptr[index+1].X() || tmpptr[index].Y()>tmpptr[index+1].Y() )
							j=maxcols;
						index++;
					}
				}
				free(tmpptr);

/*				// printout data
				for(int i=0;i<rows;i++) {
					for(int j=0;j<maxcols;j++)
						printf("(%g,%g,%g) ",ptr[i*maxcols+j].X(),ptr[i*maxcols+j].Y(),ptr[i*maxcols+j].Z());
					printf("\n");
				}
*/
				if(swap) {		// swap when y is running
					int tmp = rows;
					rows=maxcols;
					maxcols=tmp;
				}

				numrows=rows;

				// if unclear : ask for dimensions
				if(numrows <= 1 || maxcols <= 1) {
					bool ok;
					int res = QInputDialog::getInteger(i18n("Surface plot from X-Y-Z"),
						i18n("Please specify dimension in X direction : "),numrows,2,table->numRows()/2,1,&ok);
					if(ok)
						numrows=res;
					else
						return;
					cols = table->numRows()/numrows;
 				}
			}
			kdDebug()<<"	rows/cols = "<<numrows<<'/'<<maxcols<<endl;
			Graph3D *g = new Graph3D(caption(),columnTitle(i),range,SSPREADSHEET,
				type,style,symbol,ptr,maxcols,numrows);

			// use the same destination for every row
			destination=mw->addGraph3D(g,destination,type);
			kdDebug()<<"Setting destination to "<<destination<<endl;
		}
	}
}

void Spreadsheet::plotMatrix(PType type) {
	kdDebug()<<"Spreadsheet::plotMatrix() : "<<type<<endl;

	int NX = table->numCols(), NY = table->numRows();
	double *a = new double[NY*NX];

	double z, zmin=0, zmax=1;
	// TODO : skip empty lines
	// TODO : only selected rows/cols
	for(int i=0;i<NY;i++) {
		for(int j=0;j<NX;j++) {
			z = mw->formatLabel(table->text(i,j),formatItem(j));
			bool nanvalue=false;
			if (!finite(z)) { z=0; nanvalue=true; }

			if(i==0 && j==0) {
				zmin=zmax=z;
			}

			z<zmin?zmin=z:0;
			z>zmax?zmax=z:0;

			a[j+NX*i] = z;
			// TODO : mask if nanvalue
		}
	}

	// make new GraphM
	LRange range[3];
	range[0] = LRange(0,NX);
	range[1] = LRange(0,NY);
	range[2] = LRange(zmin,zmax);

	Style *style= new Style(LINESTYLE);
	Symbol *symbol= new Symbol(SNONE);

	GraphM *g = new GraphM(caption(),i18n("Matrix"),range,SSPREADSHEET,type,style,symbol,a,NX,NY);

	mw->addGraphM(g,destination,type);
}

void Spreadsheet::plotL(PType type) {
	kdDebug()<<"Spreadsheet::plotL() : "<<type<<endl;

	// if less than 2 columns selected -> select all columns
	int selected_cols=0;
	for (int s=0;s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);
#if QT_VERSION > 0x030102
		selected_cols+=ts.numCols();
#else
		selected_cols+=ts.rightCol()-ts.leftCol();
#endif
	}
	if(selected_cols<2)
		for (int i=0;i<table->numCols();i++)
			table->selectColumn(i);

	// find v,l column
	int indexv=-1, indexl=-1;
	for(int i=table->numCols()-1;i>=0;i--) {
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[LABEL]") == 0)
			indexv=i;
		if(table->isColumnSelected(i) && table->horizontalHeader()->label(i).findRev("[LABEL]") > 0)
			indexl=i;
	}

	PointL *ptr = new PointL[table->numRows()];
	// read x and y values
	double v, vmin=0,vmax=1;
	QString l;
	int numrows=0;

	// only selected rows
	QTableSelection ts = table->selection(table->currentSelection());
//	kdDebug()<<"NUM SEL = "<<table->numSelections()<<endl;
	int start=0, end=table->numRows();
	if(ts.bottomRow()>ts.topRow()) {
		start = ts.topRow();
		end =  ts.bottomRow();
	}
	kdDebug()<<"	reading ROWS from "<<start<<" to "<<end<<endl;
	for (int row = start; row < end+1; row++) {
		if(indexv == -1)
			v = (double) row + 1;
		else
			v = mw->formatLabel(table->text(row,indexv),formatItem(indexv));

		// skip empty rows
		if (table->text(row,indexl).isEmpty())
			continue;

		l = table->text(row,indexl);

		bool nanvalue=false;
		if(!finite(v)) { v=0; nanvalue=true; }

		if(row==start)
			vmin=vmax=v;

		v<vmin?vmin=v:0;
		v>vmax?vmax=v:0;

		ptr[numrows].setPointL(v,l);
		if(nanvalue)
			ptr[numrows].setMasked();
		LTableItem *item = (LTableItem *) table->item(row,indexl);
		if(item && item->Masked())
			ptr[numrows].setMasked();
		numrows++;
	}
	kdDebug()<<"	vmin/vmax = "<<(int)vmin<<' '<<(int)vmax<<endl;

	// make new GraphL
	LRange range = LRange(vmin,vmax);

	Style *style = mw->defaultStyle();
	Symbol *symbol = mw->defaultSymbol();

	GraphL *g = new GraphL(caption(),i18n("Data"),range,SSPREADSHEET,type,style,symbol,ptr,numrows);
	mw->addGraphL(g,destination,type);

	kdDebug()<<"Spreadsheet::plotL() DONE"<<endl;
}

void Spreadsheet::setTitle(QString t) {
	kdDebug()<<"Spreadsheet::setTitle() : "<<t<<endl;
	bool ok=true;
	if(t.isEmpty())
		t = QInputDialog::getText("LabPlot", i18n("Spreadsheet title : "),QLineEdit::Normal,title, &ok, this );
	if(!ok) return;

	if(!t.isEmpty()) {
		title = t.remove(QChar('&'));
		setCaption(title);
		mw->updateSheetList();
	}
}

void Spreadsheet::setNotes(QString t) {
	kdDebug()<<"Spreadsheet::setNotes()"<<endl;
	bool ok=true;
#if KDE_VERSION >= 0x030200
	if(t.isEmpty())
		t = KInputDialog::getMultiLineText("LabPlot", i18n("Spreadsheet notes : "),notes, &ok);
#endif
	if(!ok) return;

	notes = t;
}

void Spreadsheet::selectDestination() { (new DestinationDialog(mw,0))->show(); }

void Spreadsheet::addGraph2D(Graph2D *g) {
	kdDebug()<<"Spreadsheet::addGraph2D()"<<endl;
	gtype=GRAPH2D;
	if(table->numCols()<2)
		table->setNumCols(2);

	// add empty cols if necessary
	for(int i=0;i<2;i++)
		if(filledRows(table->numCols()-2)>1)
			table->setNumCols(table->numCols()+1);

	table->setNumRows(g->Number());

	Point *data = g->Data();
	table->horizontalHeader()->setLabel(table->numCols()-2 , QString( "A ")+i18n("{double}")+" [X]");
	table->horizontalHeader()->setLabel(table->numCols()-1 , QString( "B ")+i18n("{double}")+" [Y]");
	for(int i=0;i<g->Number();i++) {
		LTableItem *xitem = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].X(),'g',15));
		LTableItem *yitem = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].Y(),'g',15));
		if(data[i].Masked()) {
			xitem->setMasked();
			yitem->setMasked();
		}
		table->setItem(i, table->numCols()-2, xitem);
		table->setItem(i, table->numCols()-1, yitem);
	}
}

void Spreadsheet::addGraph3D(Graph3D *g) {
	kdDebug()<<"Spreadsheet::addGraph3D()"<<endl;
	gtype=GRAPH3D;
	if(table->numCols()<3)
		table->setNumCols(3);

	// add empty cols if necessary
	for(int i=0;i<3;i++) {
		if(filledRows(table->numCols()-3)>1) {
			table->setNumCols(table->numCols()+1);
		}
	}

	table->setNumRows(g->Number());
	kdDebug()<<"	TABLE has "<<table->numCols()<<" cols and "<<table->numRows()<<" rows"<<endl;

	Point3D *data = g->Data();
	table->horizontalHeader()->setLabel(table->numCols()-3 , QString( "A ")+i18n("{double}")+" [X]");
	table->horizontalHeader()->setLabel(table->numCols()-2 , QString( "B ")+i18n("{double}")+" [Y]");
	table->horizontalHeader()->setLabel(table->numCols()-1 , QString( "C ")+i18n("{double}")+" [Z]");
	for(int i=0;i<g->Number();i++) {
		LTableItem *xitem = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].X(),'g',15));
		LTableItem *yitem = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].Y(),'g',15));
		LTableItem *zitem = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].Z(),'g',15));
		if(data[i].Masked()) {
			xitem->setMasked();
			yitem->setMasked();
			zitem->setMasked();
		}
		table->setItem(i, table->numCols()-3, xitem);
		table->setItem(i, table->numCols()-2, yitem);
		table->setItem(i, table->numCols()-1, zitem);
	}
}

void Spreadsheet::addGraph4D(Graph4D *g) {
	kdDebug()<<"Spreadsheet::addGraph4D()"<<endl;
	gtype=GRAPH4D;
	if(table->numCols()<4)
		table->setNumCols(4);

	// add empty cols if necessary
	for(int i=0;i<4;i++)
		if(filledRows(table->numCols()-4)>1)
			table->setNumCols(table->numCols()+1);

	table->setNumRows(g->Number());

	Point4D *data = g->Data();
	table->horizontalHeader()->setLabel(table->numCols()-4 , QString( "A ")+i18n("{double}")+" [X]");
	table->horizontalHeader()->setLabel(table->numCols()-3 , QString( "B ")+i18n("{double}")+" [Y]");
	table->horizontalHeader()->setLabel(table->numCols()-2 , QString( "C ")+i18n("{double}")+" [DX]");
	table->horizontalHeader()->setLabel(table->numCols()-1 , QString( "D ")+i18n("{double}")+" [DY]");
	for(int i=0;i<g->Number();i++) {
		LTableItem *xitem = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].X(),'g',15));
		LTableItem *yitem = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].Y(),'g',15));
		LTableItem *zitem = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].Z(),'g',15));
		LTableItem *titem = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].T(),'g',15));
		if(data[i].Masked()) {
			xitem->setMasked();
			yitem->setMasked();
			zitem->setMasked();
			titem->setMasked();
		}
		table->setItem(i, table->numCols()-4, xitem);
		table->setItem(i, table->numCols()-3, yitem);
		table->setItem(i, table->numCols()-2, zitem);
		table->setItem(i, table->numCols()-1, titem);
	}
}

void Spreadsheet::addGraphM(GraphM *g) {
	kdDebug()<<"Spreadsheet::addGraphM()"<<endl;
	gtype=GRAPHM;
	int nx = g->NX();
	int ny = g->NY();
	double *data = g->Data();

	table->setNumCols(nx);
	table->setNumRows(ny);

	for (int i=0;i<ny;i++) {
		for (int j=0;j<nx;j++) {
			setText(i,j,QString::number(data[j+ny*i],'g',15));
		}
	}
}

void Spreadsheet::addGraphL(GraphL *g) {
	kdDebug()<<"Spreadsheet::addGraphL()"<<endl;
	gtype=GRAPHL;
	if(table->numCols()<2)
		table->setNumCols(2);

	// add empty cols if necessary
	for(int i=0;i<2;i++)
		if(filledRows(table->numCols()-2)>1)
			table->setNumCols(table->numCols()+1);

	table->setNumRows(g->Number());

	PointL *data = g->Data();
	table->horizontalHeader()->setLabel(table->numCols()-2 , QString( "A ")+i18n("{double}")+" [X]");
	table->horizontalHeader()->setLabel(table->numCols()-1 , QString( "B ")+i18n("{double}")+" [LABEL]");
	for(int i=0;i<g->Number();i++) {
		LTableItem *xitem = new LTableItem( table, QTableItem::OnTyping,QString::number(data[i].V(),'g',15));
		LTableItem *yitem = new LTableItem( table, QTableItem::OnTyping,data[i].L());
		if(data[i].Masked()) {
			xitem->setMasked();
			yitem->setMasked();
		}
		table->setItem(i, table->numCols()-2, xitem);
		table->setItem(i, table->numCols()-1, yitem);
	}
}

//! create Graph2D from table data
Graph2D* Spreadsheet::getGraph2D() {
	kdDebug()<<"Spreadsheet::getGraph2D()"<<endl;
	int n=table->numRows();
	Point *ptr = new Point[n];

	double ymin=0,ymax=1, xmin=0, xmax=1;
	for(int i = 0;i <n ;i++) {
		double x = table->text(i,0).toDouble();
		double y = table->text(i,1).toDouble();

		ptr[i].setPoint(x,y);
	}

	mw->calculateRanges2D(ptr,n,&xmin,&xmax,&ymin,&ymax);

	QString fun("2d data");
	QString label("data");
	LRange range[2];
	range[0] = LRange(xmin,xmax);
	range[1] = LRange(ymin,ymax);

	Graph2D *g = new Graph2D(fun,label,range,SSPREADSHEET,P2D,0,0,ptr,n);

	return g;
}

Graph3D* Spreadsheet::getGraph3D() {
	kdDebug()<<"Spreadsheet::getGraph3D() TODO"<<endl;

	int n=table->numRows();
	Point3D *ptr = new Point3D[n];

	double ymin=0,ymax=1, xmin=0, xmax=1,zmin=0,zmax=1;
	for(int i = 0;i < n ;i++) {
		double x = table->text(i,0).toDouble();
		double y = table->text(i,1).toDouble();
		double z = table->text(i,2).toDouble();

		ptr[i].setPoint(x,y,z);

	}

	mw->calculateRanges3D(ptr,n,&xmin,&xmax,&ymin,&ymax,&zmin,&zmax);

	QString fun("3d data");
	QString label("data");
	LRange range[3];
	range[0] = LRange(xmin,xmax);
	range[1] = LRange(ymin,ymax);
	range[2] = LRange(zmin,zmax);

	Graph3D *g = new Graph3D(fun,label,range,SSPREADSHEET,P2D,0,0,ptr,n);

	return g;
}

Graph4D* Spreadsheet::getGraph4D() {
	kdDebug()<<"Spreadsheet::getGraph4D() TODO"<<endl;

	int n=table->numRows();
	Point4D *ptr = new Point4D[n];

	double ymin=0,ymax=1, xmin=0, xmax=1,zmin=0,zmax=1,tmin=0,tmax=1;
	for(int i = 0;i < n ;i++) {
		double x = table->text(i,0).toDouble();
		double y = table->text(i,1).toDouble();
		double z = table->text(i,2).toDouble();
		double t = table->text(i,3).toDouble();

		ptr[i].setPoint(x,y,z,t);
	}

	mw->calculateRanges4D(ptr,n,&xmin,&xmax,&ymin,&ymax,&zmin,&zmax,&tmin,&tmax);

	QString fun("4d data");
	QString label("data");
	LRange range[4];
	range[0] = LRange(xmin,xmax);
	range[1] = LRange(ymin,ymax);
	range[2] = LRange(zmin,zmax);
	range[3] = LRange(tmin,tmax);

	Graph4D *g = new Graph4D(fun,label,range,SSPREADSHEET,P2D,0,0,ptr,n);

	return g;
}

GraphM* Spreadsheet::getGraphM() {
	kdDebug()<<"Spreadsheet::getGraphM()"<<endl;
	int nx = table->numCols();
	int ny = table->numRows();

	double *data = new double[nx*ny];

	double zmin=0,zmax=1;
	for (int i=0;i<ny;i++) {
		for (int j=0;j<nx;j++) {
			double z = table->text(i,j).toDouble();
			if (i == 0 && j == 0) {
				zmin=z;
				zmax=z;
			}

			z<zmin?zmin=z:0;
			z>zmax?zmax=z:0;

			data[j+nx*i] = z;
		}
	}

	QString fun("2d data");
	QString label("data");
	LRange range[3];
	// old : used 0..NX,0..NY
	// new : use range
	range[0] = LRange(0,nx);
	range[1] = LRange(0,ny);
	range[2] = LRange(zmin,zmax);

	Style *style = new Style(LINESTYLE);
	Symbol *symbol = new Symbol(SNONE);
	GraphM *g = new GraphM(fun,label,range,SSPREADSHEET,PSURFACE,style,symbol,data,nx,ny);

	return g;
}

//! create GraphL from table data
GraphL* Spreadsheet::getGraphL() {
	kdDebug()<<"Spreadsheet::getGraphL()"<<endl;
	//TODO
/*	int n=table->numRows();
	Point *ptr = new Point[n];

	double ymin=0,ymax=1, xmin=0, xmax=1;
	for(int i = 0;i < n ;i++) {
		double x = table->text(i,0).toDouble();
		double y = table->text(i,1).toDouble();

		ptr[i].setPoint(x,y);
	}

	mw->calculateRanges2D(ptr,n,&xmin,&xmax,&ymin,&ymax);

	QString fun("2d data");
	QString label("data");
	LRange range[2];
	range[0] = LRange(xmin,xmax);
	range[1] = LRange(ymin,ymax);

	GraphL *g = new GraphL(fun,label,range,SSPREADSHEET,P2D,0,0,ptr,n);

	return g;
*/
	return 0;
}

void Spreadsheet::cutSelection() {
	copySelection();
	clearSelection();
}

void Spreadsheet::copySelection() {
	QString text;
	for (int s=0; s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);

		for (int i=ts.topRow();i<=ts.bottomRow();i++) {
			for (int j=ts.leftCol();j<ts.rightCol();j++)
				text += table->text(i,j)+"\t";
			text += table->text(i,ts.rightCol())+"\n";
		}
	}
	// Copy text into the clipboard
	QApplication::clipboard()->setText(text);

	table->repaintContents();
}

void Spreadsheet::pasteSelection() {
	QString text = QApplication::clipboard()->text();

	QStringList rowTexts = QStringList::split ("\n",text,true);
	int rows = int(rowTexts.count())-1;
	QStringList cellTexts = QStringList::split ("\t",rowTexts[0],true);
	int cols = int(cellTexts.count());

	int top,bottom,right,left;

	QTableSelection ts = table->selection(table->currentSelection());
#if QT_VERSION > 0x030102
	if (!ts.isEmpty()) {
#else
	if (!((ts.bottomRow()-ts.topRow())*(ts.rightCol()-ts.leftCol()))) {
#endif
		top=ts.topRow();
		bottom=ts.bottomRow();
		left=ts.leftCol();
		right=ts.rightCol();
	}
	else {
		top=0;
		bottom=rows;
		left=ts.leftCol();
		right=ts.leftCol()+cols;

		int tableCols=table->numCols();
		if (right>tableCols)
			right=tableCols-1;
	}

	QStringList allCells;
	for (int i=0;i<rows;i++) {
		cellTexts=QStringList::split ("\t",rowTexts[i],true);
		for (int j=0;j<cols;j++)
			allCells<<cellTexts[j];
	}

	// currentSelection < text
	int col=right-left+1;
	if (rows>(bottom-top+1) || cols>col) {
		switch( KMessageBox::questionYesNo(this,
			i18n("The text in the clipboard is larger then your current selection!\n")+
				i18n("Do you want to adjust it to the current selection?")))  {
		case KMessageBox::Yes:
			for (int i=top;i<=bottom;i++) {
				for (int j=left;j<=right;j++)
					setText(i,j,allCells[(i-top)*cols+j-left]);
			}
			return;
			break;
		}
	}

	// insert text
	for (int i=top;i<top+rows;i++) {
		for (int j=left;j<left+cols;j++)
			setText(i,j,allCells[(i-top)*cols+(j-left)]);
	}

	table->repaintContents();
}

void Spreadsheet::clearSelection() {
	for (int s=0; s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);

		for(int i=ts.topRow();i<=ts.bottomRow();i++) {
			for(int j=ts.leftCol();j<=ts.rightCol();j++) {
				table->clearCell(i,j);
				table->updateCell(i,j);
			}
		}
	}
	table->repaintContents();
}

void Spreadsheet::selectAll() {
	for(int i=0;i<=table->numCols();i++)
		table->selectColumn(i);
}

void Spreadsheet::selectNone() {
	table->clearSelection();
}

void Spreadsheet::selectInvert() {
	int rows = table->numRows();
	bool *selected = new bool[rows];
	for(int i=0;i<rows;i++) {
		if(table->isRowSelected(i)) {
//			kdDebug()<<"	ROW "<<i+1<<" is SELECTED"<<endl;
			selected[i]=true;
		}
		else
			selected[i]=false;
	}

	table->clearSelection();
	for(int i=0;i<rows;i++) {
		if(!selected[i])
			table->selectRow(i);
	}
}

void Spreadsheet::setValues(int startrow, int endrow, QString expr) {
	if(expr.isEmpty()) {
		if(values_dialog == 0)
			values_dialog = new SpreadsheetValuesDialog(mw,table,caption());
		values_dialog->updateDialog();
		values_dialog->show();
	}
	else {
		expr.remove(QRegExp(".*="));		// remove any "xyz =" before expression
		int col = table->currentColumn();

		// save expression in config with empty header
		kdDebug()<<"	Saving entry "<<expr<<endl;
		KConfig *config = mw->Config();
		config->setGroup( "Spreadsheet" );

		QString oldexpr = config->readEntry(QString("expression%1").arg(col),"");
		if(!oldexpr.isEmpty()) {	// copy entry to > 1000
			int entry = 999;

			// find free entry
			QString tmp;
			bool expr_found = false;
			do {
				entry++;
				tmp = config->readEntry(QString("expression%1").arg(entry),"");

				if(tmp==expr) 	// already saved
					expr_found = true;

			}	while(!tmp.isEmpty() && !expr_found);

			// save old entry
			if(!expr_found) {
				kdDebug()<<"	Saving old entry as "<<entry<<endl;
				config->writeEntry(QString("expression%1").arg(entry),oldexpr);
			}
			else
				kdDebug()<<"	old entry already saved as "<<entry<<endl;
		}

		// write new entry
		config->writeEntry(QString("expression%1").arg(col),expr);

		if(endrow > table->numRows())
			table->setNumRows(endrow);

		for(int i = startrow; i <= endrow; i++) {
			QString tmp(expr.lower());
			//		kdDebug()<<"	APPLYING EXPR "<<tmp<<endl;

			// replace "col(X)" strings
			for(int j=0;j<fmin(table->numCols(),MAX_COLS);j++) {
				// build col(A)
				QString coltxt("col()");
				if(j<26)
					coltxt.insert(coltxt.length()-1,QChar(65+j));
				else
					coltxt.insert(coltxt.length()-2,QString(QChar(65+j/26-1)) + QString(QChar(65+j%26)));

				// replace col(a) with value
				if(table->text(i-1,j).isEmpty())
					tmp.replace(coltxt.lower(),"0");
				else
					tmp.replace(coltxt.lower(),table->text(i-1,j));
			}

			// replace "col(X,index)" strings
			QRegExp rx("col\\(.*\\)");		// dont use '(',')' inside expression !
			rx.setMinimal(true);

			init_table();
			QStringList list;
			while(true) {
				int pos = rx.search(tmp);
				if(pos<0)
					break;

				QString coltxt=rx.cap(0);
				//			kdDebug()<<"	FOUND : "<<coltxt<<endl;

				// find column name and index expression
				QRegExp rxc("\\w+,");
				QRegExp rxe(",.*\\)");
				rxc.setMinimal(true);
				rxc.search(coltxt);
				rxe.search(coltxt);
				QString column = rxc.cap(0).remove(',');
				QString idxexpr = rxe.cap(0).remove(',');
				idxexpr.truncate(idxexpr.length()-1);	// remove ')'
				//			kdDebug()<<"	COLUMN : "<<column<<endl;
				//			kdDebug()<<"	EXPRESSION : "<<expr<<endl;

				// calculate index
				char var[] = "i";
				assign_variable(var,i-1);
				//			kdDebug()<<"	parsed EXPRESSION : "<<expr<<endl;
				int idx = (int) parse((char *)idxexpr.latin1());

				//			kdDebug()<<"	INDEX : "<<idx<<endl;
				for(int j=0;j<fmin(table->numCols(),MAX_COLS);j++) {
					QString colname;
					if(j<26)
						colname = QChar(65+j);
					else
						colname = QString(QChar(65+j/26-1)) + QString(QChar(65+j%26));
					if(colname.lower() == column.lower()) {
						double value=0;
						if(idx>=0 && idx<table->numRows())
							value = table->text(idx,j).toDouble();
					//	kdDebug()<<"	setValue : "<<value<<endl;
						tmp.replace(coltxt,QString::number(value,'g',15));
					}
				}
			}
			delete_table();

			init_table();
			// replace "i" with index (i)
			char var[] = "i";
			assign_variable(var,i);

			//		kdDebug()<<"	PARSE EXPR "<<tmp<<endl;
			double result = parse((char *) tmp.latin1());
			//		printf("\tRESULT = %.15e\n",result);
			delete_table();

			// if(!finite(result)) result=0;		???

			if(parse_errors()>0) {
				KMessageBox::error(mw, i18n("Parse Error!\n Please check the given function."));
				return;
			}

			LTableItem *item = new LTableItem( table, QTableItem::OnTyping,QString::number(result,'g',15));
			table->setItem(i-1, col, item);
		}
	}
}

void Spreadsheet::fillRowNumber() {
	for(int i=0;i<table->numRows();i++)
		setText(i,table->currentColumn(),QString::number(i+1,'g',15));
}

void Spreadsheet::fillRandom(double max) {
	srandom(time(0));
	for(int i=0;i<table->numRows();i++)
		setText(i,table->currentColumn(),QString::number(max*random()/(double)RAND_MAX,'g',15));
}

void Spreadsheet::normSum() {
	// calculate sum
	double sum=0;
	int row=0, col = table->currentColumn();
	do {
		sum += table->text(row,col).toDouble();
		row++;
	} while(!table->text(row,0).isEmpty());

	// set value;
	row=0;
	do {
		double value = table->text(row,col).toDouble();
		setText(row,col,QString::number(value/sum,'g',15));
		row++;
	} while(!table->text(row,0).isEmpty());
}

void Spreadsheet::normMax(double maximum) {
	// calculate max
	double max=0;
	int row=0, col = table->currentColumn();
	do {
		double value = table->text(row,col).toDouble();
		if(row==0)
			max=value;
		if(max<value)
			max = value;
		row++;
	} while(!table->text(row,0).isEmpty());

	// set value;
	row=0;
	do {
		double value = table->text(row,col).toDouble();
		setText(row,col,QString::number(maximum*value/max,'g',15));
		row++;
	}while(!table->text(row,0).isEmpty());
}

void Spreadsheet::editEditor(int editor) {
	kdDebug()<<"	edit with editor "<<editor<<endl;
	KProcess* proc = new KProcess;

	switch(editor) {
	case 0:	*proc << "xterm"<<"-e"<<"vim"; break;
	case 1:	*proc << "kvim"<<"-f"; break;
	case 2:	*proc << "gvim"<<"-f"; break;
	case 3:	*proc << "kate" << "-n"; break;
	case 4:	*proc << "kwrite"; break;
	case 5:	*proc << "emacs"; break;
	case 6:	*proc << "xemacs"; break;
	case 7:	*proc << "kword"; break;
	case 8:	*proc << "soffice"; break;
	}

	// tmp file for vim;
	KTempFile *tmpfile = new KTempFile(QString::null,".dat");
	tmpfile->setAutoDelete(true);
	tmpfilename = tmpfile->name();

	// file as argument
	*proc<<tmpfilename;

	QTextStream *t = tmpfile->textStream();

	for (int i=0;i<table->numRows();i++) {
		for (int j=0;j<table->numCols();j++) {
				*t << table->text(i,j).toDouble()<<' ';
		}
		*t<<endl;
	}
	tmpfile->close();

	// TODO ; "kate" doesn't work : exit signal shortly after start (background job)
	connect(proc, SIGNAL(processExited(KProcess*)),this, SLOT(readfile(KProcess*)));
	if ( !proc->start(KProcess::NotifyOnExit) )
		KMessageBox::error( this,i18n("Could not start selected Editor."));
}

//read data from tmp file into table
void Spreadsheet::readfile(KProcess *) {
	kdDebug()<<"READING "<<tmpfilename<<endl;
	QFile file(tmpfilename);
	if ( ! file.open( IO_ReadOnly )) {
		KMessageBox::error(this,i18n("temporary data file not found!"));
		return;
	}
	QTextStream t(&file);

	int i=0;
	while(!(t.atEnd())) {
		QStringList line = QStringList::split(' ',t.readLine());
		int j=0;
		for ( QStringList::Iterator it = line.begin(); it != line.end(); ++it ) {
			setText(i,j++,(*it));
		}
		i++;
		// add additional rows
		if(i>=table->numRows())
			table->setNumRows ( table->numRows()+10 );
	}

	table->setNumRows ( i );

	file.close();
}

void Spreadsheet::convertComma() {
	int nx = table->numCols();
	int ny = table->numRows();
	for(int i=0;i<ny;i++)
		for(int j=0;j<nx;j++)
			setText(i,j,table->text(i,j).replace(QChar(','),'.'));
}

void Spreadsheet::convertFORTRAN() {
	int nx = table->numCols();
	int ny = table->numRows();
	for(int i=0;i<ny;i++)
		for(int j=0;j<nx;j++) {
			setText(i,j,table->text(i,j).replace(QChar('D'),'d'));
			setText(i,j,table->text(i,j).replace(QChar('d'),'e'));
		}
}

void Spreadsheet::transposeMatrix() {
	int nx = table->numCols();
	int ny = table->numRows();

	QString *data = new QString[nx*ny];
	for(int i=0;i<ny;i++)
		for(int j=0;j<nx;j++)
			data[j+i*nx]=table->text(i,j);

	table->setNumRows(nx);
	table->setNumCols(ny);
	setColumnNames(ny);

	for(int i=0;i<ny;i++)
		for(int j=0;j<nx;j++)
			setText(j,i,data[j+i*nx]);

	delete [] data;
}

void Spreadsheet::convertMatrixtoXYZ() {
	int nx = table->numCols();
	int ny = filledRows();

	double *data = new double[nx*ny];

	for(int i=0;i<ny;i++)
		for(int j=0;j<nx;j++)
			data[j+i*nx]=table->text(i,j).toDouble();

	table->setNumCols(3);
	table->horizontalHeader()->setLabel( 2, QString( "C ")+i18n("{double}")+" [Z]");
	table->setNumRows(nx*ny);
	for(int i=0;i<nx*ny;i++) {
		setText(i,0,QString::number(i%nx+1,'g',15));
		setText(i,1,QString::number(i/nx+1,'g',15));
		setText(i,2,QString::number(data[i],'g',15));
	}

	delete [] data;
}

void Spreadsheet::convertXYZtoMatrix() {
	// find x,y,z column
	int indexx=-1, indexy=-1, indexz=-1;
	for(int i=table->numCols();i>=0;i--) {
		if(table->horizontalHeader()->label(i).findRev("[X]")>0)
			indexx=i;
		if(table->horizontalHeader()->label(i).findRev("[Y]")>0)
			indexy=i;
		if(table->horizontalHeader()->label(i).findRev("[Z]")>0)
			indexz=i;
	}

	if(indexx == -1 || indexy == -1 || indexz == -1 ) {
		KMessageBox::error(this,i18n("Could not find X, Y and Z column!"));
		return;
	}
	kdDebug()<<"	INDEX X/Y/Z "<<indexx<<' '<<indexy<<' '<<indexz<<endl;

	// calculate dimension
	int cols = 1, maxcols=0;
	int rows=0;
	for(int i=0;i < filledRows();i++) {
//		kdDebug()<<"ROW "<<i+1<<endl;
		if(i == filledRows() - 1) {
			rows++;
			maxcols<cols?maxcols=cols:0;
//			kdDebug()<<"	ROW "<<rows<<" has "<<cols<<" columns"<<endl;
		}
		else if(table->text(i,indexx).toDouble() > table->text(i+1,indexx).toDouble() ||
				      table->text(i,indexy).toDouble() > table->text(i+1,indexy).toDouble() ) {
			rows++;
			maxcols<cols?maxcols=cols:0;
//			kdDebug()<<"	ROW "<<rows<<" has "<<cols<<" columns"<<endl;
			cols=1;
		}
		else
			cols++;
	}

	// convert data and create new ptr
	kdDebug()<<"	rows/cols = "<<rows<<'/'<<maxcols<<endl;
	double *data = new double[rows*maxcols];
	for(int i=0;i<rows;i++)
		for(int j=0;j<maxcols;j++)
			data[i*maxcols+j] = 0;

	int index=0;
	for(int i=0;i<rows;i++) {
		for(int j=0;j<maxcols;j++) {
//			kdDebug()<<tmpptr[index].X()<<' '<<tmpptr[index+1].X()<<endl;
//			kdDebug()<<"SET index "<<index<<" to i*maxcols+j="<<i*maxcols+j<<" : "<<tmpptr[index].X()<<endl;
			data[i*maxcols+j] = table->text(index,indexz).toDouble();
			if(index == filledRows()-1 ||
						table->text(index,indexx).toDouble() > table->text(index+1,indexx).toDouble() ||
						table->text(index,indexy).toDouble() > table->text(index+1,indexy).toDouble() )
				j=maxcols;
			index++;
		}
	}

	// build table
	table->setNumCols(maxcols);
	table->setNumRows(rows);
	table->selectColumn(-1);

	setColumnNames(maxcols);

	// fill table
	for(int i=0;i<rows;i++)
		for(int j=0;j<maxcols;j++)
			setText(i,j,QString::number(data[i*maxcols+j],'g',15));

	delete [] data;
}

void Spreadsheet::convertColumntoMatrix() {
	kdDebug()<<"Spreadsheet::convertColumntoMatrix()"<<endl;
	// get selected column and fill tmp vector
	QTableSelection ts = table->selection(0);
	int col = ts.leftCol(), rows = table->numRows();
#ifdef HAVE_SOLARIS
#warning integer constant expression : max rows = 65535
	double tmp[65535];
#else
	double tmp[rows];
#endif
	kdDebug()<<"	selected "<<rows<<" rows from column "<<col<<endl;

	if(col==-1) col=0;

	for(int i=0;i<rows;i++)
		tmp[i]=table->text(i,col).toDouble();

	// dialog : "split after n rows. n=..."
	bool ok;
	int n = QInputDialog::getInteger(i18n("Convert Column -> Matrix"),i18n("split after row"),100,1,INF,1,&ok);
	if(!ok)
		return;
	kdDebug()<<"	split after row "<<n<<endl;

	// do conversion
	table->setNumRows(n);
	int entry=0, tmpcol=-1;
	while(entry<rows) {
		int row=entry%n;
		if(row==0) {
			tmpcol++;
			if(tmpcol>=table->numCols())
				addColumn();
		}
		setText(row,tmpcol,QString::number(tmp[entry],'g',15));

		entry++;
	}
}

ExportDialog* Spreadsheet::exportData() {
	ExportDialog *dialog = new ExportDialog(mw,"spreadsheet",-1);
	dialog->show();
	return dialog;
}

void Spreadsheet::selectColumns(int left, int right) {
	if(right==-1) right=left;

	kdDebug()<<"selecting columns "<<left<<" to "<<right<<endl;
	table->selectCells(0,left,table->numRows(),right);
}

void Spreadsheet::selectRows(int top, int bottom){
	if(bottom==-1) bottom=top;

	kdDebug()<<"selecting rows "<<top<<" to "<<bottom<<endl;
	table->selectCells(top,0,bottom,table->numCols());
}

// delete all currently selected rows
void Spreadsheet::deleteRows() {
	kdDebug()<<"Spreadsheet::deleteRows()"<<endl;
	table->setUpdatesEnabled(false);

	// much slower on SUSE 9.1 !
#if defined(HAVE_SGI_STL) && (QT_VERSION != 0x030301)
	vector<int> tmp;
	for (int s=0; s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);
		kdDebug()<<"removing row "<<ts.topRow()<<" to "<<ts.bottomRow()<<endl;

		for (int row=ts.topRow();row<=ts.bottomRow();tmp.push_back(row++))
			;
	}

	unsigned int rowcount = tmp.size();
	if (rowcount > 0) {
		// sort temporary array to meet QTable::removeRows() requirements
		stable_sort (tmp.begin(), tmp.end());

		QMemArray<int> selected_rows(rowcount);
		for (unsigned int i=0; i<rowcount; i++)
			selected_rows[i] = tmp[i];

		table->removeRows(selected_rows);
	}
#else
	int rows=0;
	for (int s=0; s < table->numSelections(); s++) {
		QTableSelection ts = table->selection(s);
		kdDebug()<<"	removing rows "<<ts.topRow()+1<<" to "<<ts.bottomRow()+1<<endl;

		for (int row = ts.topRow(); row <= ts.bottomRow(); row++) {
			// move the row to the end
			for (int i = row-rows; i < table->numRows() - 1; i++)
				table->swapRows(i, i + 1);
			rows++;
		}
	}
	table->setNumRows(table->numRows() - rows);
	table->clearSelection();
#endif

	table->setCurrentCell(0,0);
	table->setUpdatesEnabled(true);
	table->repaintContents();
}

void Spreadsheet::addColumn() {
	int col = table->numCols();

	// add as many columns as selected
	QTableSelection ts = table->selection(table->currentSelection());
#if QT_VERSION > 0x030102
	int nr = ts.numCols();
#else
	int nr = ts.rightCol()-ts.leftCol();
#endif
	if(nr == 0)
		nr=1;
	else
		col = ts.rightCol()+1;	// insert right of selection

	table->insertColumns(col,nr);
	for(int i=0; i< nr; i++) {
		int ncol=col+i;
		if(ncol < 26)
			table->horizontalHeader()->setLabel( ncol, QChar(ncol+65)+' '+i18n("{double}")+" [Y]");
		else
			table->horizontalHeader()->setLabel( ncol,
				QString(QChar(65+ncol/26-1)) + QString(QChar(65+ncol%26)) + ' ' + i18n("{double}")+" [Y]");

		// fill with LTableItems
		for(int j=0;j<table->numRows();j++) {
			LTableItem *it = new LTableItem( table, QTableItem::OnTyping,"");
			table->setItem( j, ncol, it);
		}
	}

	// TODO : enough space right of all columns
/*
//	table->horizontalHeader()->setClickEnabled(false);
//	table->horizontalHeader()->setTracking(false);
//	table->horizontalScrollBar()->center(table->contentsWidth(),0);
	if(table->visibleWidth() < table->contentsWidth()) {
		table->resizeContents(table->contentsWidth()+150,table->contentsHeight());
		table->repaintContents();
	}
*/

	table->repaintContents();
}

// delete all currently selected columns
void Spreadsheet::deleteColumns() {
	kdDebug()<<"Spreadsheet::deleteRows()"<<endl;
	table->setUpdatesEnabled(false);

#ifdef HAVE_SGI_STL
	vector<int> tmp;
	for (int s=0; s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);
		kdDebug()<<"removing column "<<ts.leftCol()<<" to "<<ts.rightCol()<<endl;

		for (int col=ts.leftCol();col<=ts.rightCol();tmp.push_back(col++))
			;
	}

	unsigned int colcount = tmp.size();
	if (colcount > 0) {
		// sort temporary array to meet QTable::removeColumns() requirements
		stable_sort (tmp.begin(), tmp.end());

		QMemArray<int> selected_cols(colcount);
		for (unsigned int i=0; i<colcount; i++)
			selected_cols[i] = tmp[i];

		table->removeColumns(selected_cols);
	}
#else
	int cols=0;
	for (int s=0; s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);
		kdDebug()<<"removing column "<<ts.leftCol()<<" to column "<<ts.rightCol()<<endl;

		for (int col = ts.leftCol(); col <= ts.rightCol(); col++) {
			// move the col to the right
			for (int i = col-cols; i < table->numCols() - 1; i++)
				table->swapColumns(i, i + 1,true);
			cols++;
		}
	}
	table->setNumCols(table->numCols() - cols);
	table->clearSelection();
#endif

	table->setCurrentCell(0,0);
	table->setUpdatesEnabled(true);
	table->repaintContents();
}

void Spreadsheet::maskSelection() {
	for (int s=0; s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);

		for(int i=ts.topRow();i<=ts.bottomRow();i++) {
			for(int j=ts.leftCol();j<=ts.rightCol();j++) {
				LTableItem *item = (LTableItem *) table->item(i,j);

				if(item->Masked())
					item->setMasked(false);
				else
					item->setMasked();
			}
		}
	}
}

void Spreadsheet::toggleMask() {
	int nx = table->numCols();
	int ny = table->numRows();

	for(int i=0;i<ny;i++) {
		for(int j=0;j<nx;j++) {
			LTableItem *item = (LTableItem *) table->item(i,j);
			if(item->Masked())
				item->setMasked(false);
			else
				item->setMasked();
		}
	}
	table->repaintContents();
}

void Spreadsheet::unMask() {
	int nx = table->numCols();
	int ny = table->numRows();

	for(int i=0;i<ny;i++) {
		for(int j=0;j<nx;j++) {
			LTableItem *it = new LTableItem( table, QTableItem::OnTyping,table->text(i,j));
			table->setItem( i, j, it);
		}
	}
}

void Spreadsheet::maskNthRow() {
	bool ok;
	int n = QInputDialog::getInteger(i18n("Mask every n-th row"),i18n("n = "),10,1,INF,1,&ok);
	if(!ok) return;

	LTableItem *it;
	for(int i=0;i<table->numRows();i++) {
		if(i%n==0) {
			for(int j=0;j<table->numCols();j++) {
				it = new LTableItem( table, QTableItem::OnTyping,table->text(i,j));
				it->setMasked();
				table->setItem( i, j, it);
			}
		}
	}
}

void Spreadsheet::maskFirstRow() {
	bool ok;
	int n = QInputDialog::getInteger(i18n("Mask all but n-th row"),i18n("n = "),10,1,INF,1,&ok);
	if(!ok) return;

	LTableItem *it;
	for(int i=0;i<table->numRows();i++) {
		if(i%n) {
			for(int j=0;j<table->numCols();j++) {
				it = new LTableItem( table, QTableItem::OnTyping,table->text(i,j));
				it->setMasked();
				table->setItem( i, j, it);
			}
		}
	}
}

void Spreadsheet::sortAscending() {
	ascending=true;
	sort();
}

void Spreadsheet::sortDescending() {
	ascending=false;
	sort();
}

void Spreadsheet::sort() {
	for (int s=0; s<table->numSelections();s++) {
		QTableSelection ts = table->selection(s);
		if(ts.topRow()<ts.bottomRow())
			qsort(ts.topRow(),ts.bottomRow());
		else
			qsort(0,table->numRows()-1);
	}
	table->clearSelection();
	table->repaintContents();
}

void Spreadsheet::qsort(int s, int e) {
	if(e>s+1) {
		int col = table->currentColumn();
		double mid = table->text((s+e)/2,col).toDouble();

		int i=s, j=e;
		while (i<j) {
			if(ascending) {
				while (table->text(i,col).toDouble() < mid) i++;
				while (mid < table->text(j,col).toDouble())  j--;
			}
			else {
				while (table->text(i,col).toDouble() > mid) i++;
				while (mid > table->text(j,col).toDouble())  j--;
			}

			if(i<j) {
				table->swapRows(i,j);
				i++;j--;
			}
		}

		qsort(s,j);
		qsort(i,e);
	}
}

void Spreadsheet::Statistics(bool col) {
	Spreadsheet *s = mw->newSpreadsheet();

	s->setTitle(i18n("Statistics of ")+Title());
	QTable *t = s->Table();
	t->setNumCols(8);
	if(col) {
		t->horizontalHeader()->setLabel(0 , i18n( "Column Title ")+i18n("{}")+" [LABEL]");
		t->horizontalHeader()->setLabel(1 , i18n( "Rows ")+i18n("{double}")+" [Y]");
	}
	else {
		t->horizontalHeader()->setLabel(0 , i18n( "Row ")+i18n("{double}")+" [X]");
		t->horizontalHeader()->setLabel(1 , i18n( "Columns ")+i18n("{double}")+" [Y]");
	}
	t->horizontalHeader()->setLabel(2 , i18n( "Max ")+i18n("{double}")+" [Y]");
	t->horizontalHeader()->setLabel(3 , i18n( "Max @ ")+i18n("{double}")+" [Y]");
	t->horizontalHeader()->setLabel(4 , i18n( "Min ")+i18n("{double}")+" [Y]");
	t->horizontalHeader()->setLabel(5 , i18n( "Min @ ")+i18n("{double}")+" [Y]");
	t->horizontalHeader()->setLabel(6 , i18n( "Sum ")+i18n("{double}")+" [Y]");
	t->horizontalHeader()->setLabel(7 , i18n( "Mean ")+i18n("{double}")+" [Y]");
#ifdef HAVE_GSL
	t->setNumCols(14);
	t->horizontalHeader()->setLabel(8 , i18n( "Standard Deviation ")+i18n("{double}")+" [Y]");
	t->horizontalHeader()->setLabel(9 , i18n( "Variance ")+i18n("{double}")+" [Y]");
	t->horizontalHeader()->setLabel(10 , i18n( "Absolute Deviation ")+i18n("{double}")+" [Y]");
	t->horizontalHeader()->setLabel(11 , i18n( "Skewness ")+i18n("{double}")+" [Y]");
	t->horizontalHeader()->setLabel(12 , i18n( "Kurtosis ")+i18n("{double}")+" [Y]");
	t->horizontalHeader()->setLabel(13 , i18n( "Lag-1 Autocorrelation ")+i18n("{double}")+" [Y]");
#endif

	int cols=0;
	if(col) {	// column stats
		for (int i=0; i<table->numCols();i++) {
			if (table->isColumnSelected(i)) {
				t->setText(cols,0,columnTitle(i));
				int N = filledRows(i);
				t->setText(cols,1,QString::number(N));
				double *data = new double[N], sum=0,max=0,min=0;
				int maxi=0, mini=0;

				for(int j=0;j<N;j++) {
					double v = table->text(j,i).toDouble();
					data[j] = v;
					if(j==0)
						max=min=v;
					if(v>max) { max=v; maxi=j; }
					if(v<min) { min=v; mini=j; }
					sum += v;
				}

				t->setText(cols,2,QString::number(max));	// max
				t->setText(cols,3,QString::number(maxi+1));	// max i
				t->setText(cols,4,QString::number(min));	// min
				t->setText(cols,5,QString::number(mini+1));	// min i
				t->setText(cols,6,QString::number(sum));	// sum
				t->setText(cols,7,QString::number(sum/N));	// mean
#ifdef HAVE_GSL
				t->setText(cols,8,QString::number(gsl_stats_sd(data,1,N)));
				t->setText(cols,9,QString::number(gsl_stats_variance(data,1,N)));
				t->setText(cols,10,QString::number(gsl_stats_absdev(data,1,N)));
				t->setText(cols,11,QString::number(gsl_stats_skew(data,1,N)));
				t->setText(cols,12,QString::number(gsl_stats_kurtosis(data,1,N)));
				t->setText(cols,13,QString::number(gsl_stats_lag1_autocorrelation(data,1,N)));
#endif
				cols++;

				delete [] data;
			}
		}
	}
	else {	// row stats
		for (int i=0; i<table->numRows();i++) {
			if (table->isRowSelected(i)) {
				t->setText(cols,0,QString::number(i+1));
				int N = table->numCols();
				t->setText(cols,1,QString::number(N));

				double *data = new double[N], sum=0,max=0,min=0;
				int maxi=0, mini=0;

				for(int j=0;j<N;j++) {
					double v = table->text(i,j).toDouble();
					data[j] = v;
					if(j==0)
						max=min=v;
					if(v>max) { max=v; maxi=j; }
					if(v<min) { min=v; mini=j; }
					sum += v;
				}

				t->setText(cols,2,QString::number(max));	// max
				t->setText(cols,3,QString::number(maxi+1));	// max i
				t->setText(cols,4,QString::number(min));	// min
				t->setText(cols,5,QString::number(mini+1));	// min i
				t->setText(cols,6,QString::number(sum));	// sum
				t->setText(cols,7,QString::number(sum/N));	// mean
#ifdef HAVE_GSL
				t->setText(cols,8,QString::number(gsl_stats_sd(data,1,N)));
				t->setText(cols,9,QString::number(gsl_stats_variance(data,1,N)));
				t->setText(cols,10,QString::number(gsl_stats_absdev(data,1,N)));
				t->setText(cols,11,QString::number(gsl_stats_skew(data,1,N)));
				t->setText(cols,12,QString::number(gsl_stats_kurtosis(data,1,N)));
				t->setText(cols,13,QString::number(gsl_stats_lag1_autocorrelation(data,1,N)));
#endif
				cols++;

				delete [] data;
			}
		}
	}

	t->setNumRows(cols);
}

void Spreadsheet::setProperties(QString label, int type, int format, int rows) {
	if(label.isEmpty())
		(new SpreadsheetPropertiesDialog(mw,table,caption()))->show();
	else {
		table->setNumRows(rows);
		table->horizontalHeader()->setLabel(table->currentColumn(),label +' '+
			formatList[format]+' '+columnTypeItems[type]);
	}
}
